from torchvision.models.resnet import resnet18,resnet50,resnet34,resnet101,resnet152,resnext101_32x8d,resnext50_32x4d
import torch
from torch import optim,nn
from torch.utils.data import dataloader
from torchvision import transforms,datasets
import os,shutil,glob
import tqdm
def main():

    train(
        data_dir='/home/ars/sda5/data/chaoyuan/datasets/classify_datasets/文档方向分类',
        model_save_path='model.pkl',
        model_type='resnet18',
        num_epochs=50,
        input_size=(224,224),
        # input_size=(448,448),
        batch_size=32,
    )

resnet_models=dict(
    resnet18=resnet18,resnet34=resnet34,resnet50=resnet50,
    resnet101=resnet101,resnet152=resnet152,resnext50_32x4d=resnext50_32x4d,resnext101_32x8d=resnext101_32x8d
)
def train(
        data_dir=None,train_dir=None,batch_size=32,num_epochs=50,patience=50,num_classes=None,input_size=(224,224),data_transforms=None,val_dir=None,model_save_dir=None,model_save_path=None,pretrained_model_path=None,model_type='resnet50',use_default_pretrained_or_download=True
):
    data_transforms = data_transforms or  {
        'train': transforms.Compose([
            transforms.RandomResizedCrop(input_size,scale=(0.3,1)),
            # transforms.RandomHorizontalFlip(),
            # transforms.ColorJitter(brightness=0.5, contrast=0.5, hue=0.5),
            transforms.ColorJitter(brightness=0.2,contrast=0.2,saturation=0.2,hue=0.2),
            transforms.RandomRotation(10),
            # transforms.RandomAffine(20),
            transforms.ToTensor(),
            # transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])
            transforms.Normalize([0.5, 0.5, 0.5], [0.229, 0.224, 0.225])
        ]),
        'val': transforms.Compose([
            transforms.Resize(input_size),
            # transforms.CenterCrop(224),
            transforms.ToTensor(),
            transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])
        ]),
    }
    if not train_dir:
        train_dir=data_dir+'/train'
    if not val_dir:
        val_dir=data_dir+'/val'
    if not num_classes:
        num_classes=len(os.listdir(train_dir))
    if not model_save_path:
        model_save_path=model_save_dir+'/model.pkl'
    if pretrained_model_path:
        model=torch.load(pretrained_model_path)
    else:
        model_type=resnet_models[model_type]
        model=model_type(pretrained=use_default_pretrained_or_download)
        in_features=model.fc.in_features
        model.fc=torch.nn.Linear(in_features,num_classes)
    device=torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')
    model.to(device)

    image_datasets={'train':datasets.ImageFolder(train_dir,transform=data_transforms['train']),
                    'val':datasets.ImageFolder(val_dir,transform=data_transforms['val'])}
    dataloaders={x:dataloader.DataLoader(image_datasets[x],batch_size=batch_size,shuffle=True,num_workers=4) for x in ['train','val']}
    optimizer = optim.SGD(model.parameters(), lr=0.001, momentum=0.9)
    # optimizer=optim.Adam(model.parameters(),lr=0.001)
    # scheduler = optim.lr_scheduler.StepLR(optimizer,  step_size=7, gamma=0.1)
    criterion=nn.CrossEntropyLoss()
    best_acc = 0.0
    bad_epochs=0
    print("###############Start Training###############")
    for epoch in range(num_epochs):
        log_info=['Epoch:%s'%(epoch)]
        for phase in ['train','val']:
            if phase=='train':
                model.train()
            else:
                model.eval()
            running_loss = 0.0
            running_corrects = 0
            # Iterate over data.

            for inputs, labels in tqdm.tqdm(dataloaders[phase]):
                inputs = inputs.to(device)
                labels = labels.to(device)
                # zero the parameter gradients
                optimizer.zero_grad()

                # forward
                # track history if only in train
                with torch.set_grad_enabled(phase == 'train'):
                    outputs = model(inputs)
                    _, preds = torch.max(outputs, 1)
                    loss = criterion(outputs, labels)

                    # backward + optimize only if in training phase
                    if phase == 'train':
                        loss.backward()
                        optimizer.step()

                # statistics
                running_loss += loss.item() * inputs.size(0)
                running_corrects += torch.sum(preds == labels.data)
            # if phase == 'train':
            #     scheduler.step()

            epoch_loss = running_loss / len(dataloaders[phase].dataset)
            epoch_acc = running_corrects.double() / len(dataloaders[phase].dataset)

            log_info.append('%s[loss:%.4f\taccuracy:%.4f]'%(
                phase, epoch_loss, epoch_acc))

            # deep copy the model
            if phase == 'val':
                if epoch_acc > best_acc:
                    bad_epochs=0
                    best_acc = float(epoch_acc)
                    torch.save(model,model_save_path)
                    print('\nNew best model saved to %s ,accuracy: %s ,loss %s '%(model_save_path,epoch_acc,epoch_loss))
                else:
                    if best_acc>=0.5:
                        bad_epochs+=1
                    if bad_epochs>=patience:
                        print("Val accuracy stopped improving, stop training, best accuracy: %s , model was saved at %s"%(best_acc,model_save_path))
                        return
        print('\t'.join(log_info))



if __name__ == '__main__':
    main()




import cv2
import numpy as np
import random,math
# 几何变换 geometrical

# 缩放
def resize(img,size):
    return cv2.resize(img,size)
def rescale(img,scale):
    if isinstance(scale,(tuple,list)):
        assert len(scale)==2
        scaleX,scaleY=scale
    else:
        scaleX=scaleY=scale
    h,w=img.shape[:2]
    h,w=int(h*scaleY),int(w*scaleX)
    return cv2.resize(img,(w,h))
# crop
def randomCrop(img,size):
    h,w=img.shape[:2]
    nw,nh=size
    spaceX=w-nw
    spaceY=h-nh
    ofx=random.randint(0,spaceX+1)
    ofy=random.randint(0,spaceY+1)
    img=img[ofy:ofy+spaceY,ofx:ofx+spaceX]
    return img
# flip
def herizontal_flip(img):
    return cv2.flip(img,1)
def vertical_flip(img):
    return cv2.flip(img,0)
# shift
def shift(image,offset):
    x,y=offset
    M = np.float32([[1, 0, x], [0, 1, y]])  # 4  //X轴移动x, Y中移动y
    shifted = cv2.warpAffine(image, M, (image.shape[1], image.shape[0]))  # 5
    return shifted
# rotate

def rotate(image, angle):
    # 获取图像的尺寸
    # 旋转中心
    (h, w) = image.shape[:2]
    (cx, cy) = (w / 2, h / 2)
    # 设置旋转矩阵
    M = cv2.getRotationMatrix2D((cx, cy), angle, 1.0)
    cos = np.abs(M[0, 0])
    sin = np.abs(M[0, 1])
    # 计算图像旋转后的新边界
    nW = int((h * sin) + (w * cos))
    nH = int((h * cos) + (w * sin))
    # 调整旋转矩阵的移动距离（t_{x}, t_{y}）
    M[0, 2] += (nW / 2) - cx
    M[1, 2] += (nH / 2) - cy
    return cv2.warpAffine(image, M, (nW, nH))


def rotate_box(bboxes, angle, cx, cy, h, w):
    bboxes=np.array(bboxes)
    def ltrb_to_four_corners(box):
        l, t, r, b = box
        # box2=[r,t,r,t,r,b,l,b]
        box2 = [l, t, r, t, r, b, l, b]
        return np.array(box2)

    def four_coners_to_ltrb(box, sort=False):
        x1, y1, x2, y2, x4, y4, x3, y3 = box
        l = min(x1, x3)
        r = max(x2, x4)
        t = min(y1, y2)
        b = max(y3, y4)
        if sort:
            newbox = [min(l, r), min(t, b), max(l, r), max(t, b)]
        else:
            newbox = [l, t, r, b]
        return np.array(newbox)

    corners = np.array([ltrb_to_four_corners(box) for box in bboxes])
    corners = corners.reshape(-1, 2)
    corners = np.hstack((corners, np.ones((corners.shape[0], 1), dtype=type(corners[0][0]))))

    M = cv2.getRotationMatrix2D((cx, cy), angle, 1.0)

    cos = np.abs(M[0, 0])
    sin = np.abs(M[0, 1])

    nW = int((h * sin) + (w * cos))
    nH = int((h * cos) + (w * sin))
    # adjust the rotation matrix to take into account translation
    M[0, 2] += (nW / 2) - cx
    M[1, 2] += (nH / 2) - cy
    # Prepare the vector to be transformed
    calculated = np.dot(M, corners.T).T

    calculated = calculated.reshape(-1, 8)
    calculated=[four_coners_to_ltrb(box) for box in calculated]
    return calculated

def rotate_img_and_bboxes(img,bboxes,angle):
    h,w=img.shape[:2]
    img=rotate(img,angle)
    bboxes=rotate_box(bboxes,angle,w//2,h//2,h,w)
    return img,bboxes

def random_rotate_img_and_bboxes(img,bboxes,angle=30):
    if isinstance(angle,(tuple,list)):
        lower,higher=angle
    else:
        lower,higher=(-angle,angle)
    angle=random.randrange(lower,higher)
    return rotate_img_and_bboxes(img,bboxes,angle)
# affine
def affine(img,pnts1,pnts2,size=None):
    pnts1=np.array(pnts1)
    pnts2=np.array(pnts2)
    M=cv2.getAffineTransform(pnts1,pnts2)
    img=cv2.warpAffine(img,M,size)
    return img
def randomAffine(img):
    pass
# perspective
def perspective(img,pnts1,pnts2,size=None):
    pnts1=np.array(pnts1)
    pnts2=np.array(pnts2)
    M=cv2.getPerspectiveTransform(pnts1,pnts2)
    img=cv2.warpPerspective(img,M,size)
    return img
def randomPerspective(img):
    pass
# 环形图像展开成为长条形
def get_huan_by_circle(img, circle_center, radius, radius_width):
    black_img = np.zeros((radius_width, int(2 * radius * math.pi), 3), dtype='uint8')
    for row in range(0, black_img.shape[0]):
        for col in range(0, black_img.shape[1]):
            theta = math.pi * 2 / black_img.shape[1] * (col + 1)
            rho = radius - row - 1
            p_x = int(circle_center[0] + rho * math.cos(theta) + 0.5)
            p_y = int(circle_center[1] - rho * math.sin(theta) + 0.5)
            black_img[row, col, :] = img[p_y, p_x, :]
    return black_img
# distort


# 光学变换 optical
def adjust_hue(img,value):
    turn_green_hsv=cv2.cvtColor(img,cv2.COLOR_BGR2HSV)
    turn_green_hsv[:, :, 0] = (turn_green_hsv[:, :, 0] + value) % 255
    turn_green_img = cv2.cvtColor(turn_green_hsv, cv2.COLOR_HSV2BGR)
    return turn_green_img
def adjust_lightness(img,r):
    darker_hsv=cv2.cvtColor(img,cv2.COLOR_BGR2HSV)
    darker_hsv[:, :, 2] = r * darker_hsv[:, :, 2]
    darker_img = cv2.cvtColor(darker_hsv, cv2.COLOR_HSV2BGR)
    return darker_img

def adjust_saturation(img,r):
    colorless_hsv=cv2.cvtColor(img,cv2.COLOR_BGR2HSV)
    colorless_hsv[:, :, 1] = r * colorless_hsv[:, :, 1]
    colorless_img = cv2.cvtColor(colorless_hsv, cv2.COLOR_HSV2BGR)
    return colorless_img
# gamma trans
def gamma_trans(img, gamma):
    # 感觉越大越亮
    # 具体做法是先归一化到1，然后gamma作为指数值求出新的像素值再还原
    gamma_table = [np.power(x / 255.0, gamma) * 255.0 for x in range(256)]
    gamma_table = np.round(np.array(gamma_table)).astype(np.uint8)
    # 实现这个映射用的是OpenCV的查表函数
    return cv2.LUT(img, gamma_table)

# PCA jitter
def PCA_Jittering(img):
    img = np.asanyarray(img, dtype='float32')
    img = img / 255.0
    img_size = img.size // 3
    img1 = img.reshape(img_size, 3)
    img1 = np.transpose(img1)
    img_cov = np.cov([img1[0], img1[1], img1[2]])
    lamda, p = np.linalg.eig(img_cov)
    p = np.transpose(p)
    alpha1 = random.gauss(0, 1)
    alpha2 = random.gauss(0, 1)
    alpha3 = random.gauss(0, 1)
    v = np.transpose((alpha1 * lamda[0], alpha2 * lamda[1], alpha3 * lamda[2]))
    add_num = np.dot(p, v)
    img2 = np.array([img[:, :, 0] + add_num[0], img[:, :, 1] + add_num[1], img[:, :, 2] + add_num[2]])
    img2 = np.swapaxes(img2, 0, 2)
    img2 = np.swapaxes(img2, 0, 1)
    return img2
# light
def equalizeHist(img):
    imgYUV = cv2.cvtColor(img, cv2.COLOR_BGR2YCrCb)
    # cv2.imshow("src", img)
    channelsYUV = cv2.split(imgYUV)
    channelsYUV[0] = cv2.equalizeHist(channelsYUV[0])
    channels = cv2.merge(channelsYUV)
    result = cv2.cvtColor(channels, cv2.COLOR_YCrCb2BGR)
    return result
# adjust resolution


# blur
def motion_blur(image, degree=12, angle=45):
    image = np.array(image)
    # 这里生成任意角度的运动模糊kernel的矩阵， degree越大，模糊程度越高
    M = cv2.getRotationMatrix2D((degree / 2, degree / 2), angle, 1)
    motion_blur_kernel = np.diag(np.ones(degree))
    motion_blur_kernel = cv2.warpAffine(motion_blur_kernel, M, (degree, degree))

    motion_blur_kernel = motion_blur_kernel / degree
    blurred = cv2.filter2D(image, -1, motion_blur_kernel)
    # convert to uint8
    cv2.normalize(blurred, blurred, 0, 255, cv2.NORM_MINMAX)
    blurred = np.array(blurred, dtype=np.uint8)
    return blurred

def guassion_blur(img,kernel_size=(3,3),sigmaX=0,sigmaY=0):
    img=cv2.GaussianBlur(img,kernel_size=kernel_size,sigmaX=sigmaX,sigmaY=sigmaY)
    return img

# noise
def sp_noise(image,prob):
  '''
  添加椒盐噪声
  prob:噪声比例
  '''
  output = np.zeros(image.shape,np.uint8)
  thres = 1 - prob
  for i in range(image.shape[0]):
    for j in range(image.shape[1]):
      rdn = random.random()
      if rdn < prob:
        output[i][j] = 0
      elif rdn > thres:
        output[i][j] = 255
      else:
        output[i][j] = image[i][j]
  return output
def gaussion_noise(image, mean=0, var=0.001):
  '''
    添加高斯噪声
    mean : 均值
    var : 方差
  '''
  image = np.array(image/255, dtype=float)
  noise = np.random.normal(mean, var ** 0.5, image.shape)
  out = image + noise
  if out.min() < 0:
    low_clip = -1.
  else:
    low_clip = 0.
  out = np.clip(out, low_clip, 1.0)
  out = np.uint8(out*255)
  #cv.imshow("gasuss", out)
  return out
# random color

# randome **




# ##################################
def get_enclosing_box(corners):
    # corners
    """Get an enclosing box for ratated corners of a bounding box

    Parameters
    ----------

    corners : numpy.ndarray
        Numpy array of shape `N x 8` containing N bounding boxes each described by their
        corner co-ordinates `x1 y1 x2 y2 x3 y3 x4 y4`

    Returns
    -------

    numpy.ndarray
        Numpy array containing enclosing bounding boxes of shape `N X 4` where N is the
        number of bounding boxes and the bounding boxes are represented in the
        format `x1 y1 x2 y2`

    """
    x_ = corners[:, [0, 2, 4, 6]]
    y_ = corners[:, [1, 3, 5, 7]]

    xmin = np.min(x_, 1).reshape(-1, 1)
    ymin = np.min(y_, 1).reshape(-1, 1)
    xmax = np.max(x_, 1).reshape(-1, 1)
    ymax = np.max(y_, 1).reshape(-1, 1)

    final = np.hstack((xmin, ymin, xmax, ymax, corners[:, 8:]))

    return final



import torch
from torchvision import transforms
import os,shutil,glob
import cv2
from PIL import Image

class Predictor:
    def __init__(self, model_path='model.pkl', input_size=(224,224),classes=None,classes_path=None,transform=None):
        self.model_path = model_path
        self.model = torch.load(model_path)
        self.device = torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')
        self.model.to(self.device)

        transform = transform or  transforms.Compose([
            transforms.Resize(input_size),
            transforms.CenterCrop(input_size),
            transforms.ToTensor(),
            transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])
        ])
        self.transform=transform
        if classes_path:
            classes=open(classes_path).read().strip().split('\n')
        self.classes=classes
    def predict(self,img):
        im=img
        im = self.transform(im).float()
        im = torch.tensor(im, requires_grad=False)
        im = im.unsqueeze(0)
        im = im.to(self.device)
        y = self.model(im)
        y = torch.argmax(y).cpu().int()
        y = int(y)
        return self.classes[y]
    def predict_from_file(self,fp):
        img=Image.open(fp)
        return self.predict(img)
    def test_dir(self,dir,out_dir,verbose=True,font_path=None,font_size=64):
        from wpkit.cv.utils import mark_img,set_font_path
        if not font_path:
            if os.path.exists('msyh.ttf'):
                font_path='msyh.ttf'
        if font_path:
            set_font_path(font_path)
        fs = glob.glob(dir+'/*.jpg')
        res = {}
        if os.path.exists(out_dir):
            shutil.rmtree(out_dir)
        os.makedirs(out_dir)
        for i, f in enumerate(fs):
            name = os.path.basename(f)
            y = self.predict_from_file(f)
            img=Image.open(f)
            img=mark_img(img,y,font_size=font_size)
            f2=out_dir+'/'+os.path.basename(f)
            img.save(f2)
            if verbose:
                print(i, f, y)
        return res
    def predict_dir(self,dir,verbose=True):
        fs=glob.glob(dir+'/*.jpg')
        res={}
        for i,f in enumerate(fs):
            name=os.path.basename(f)
            y=self.predict_from_file(f)
            res[name]=y
            if verbose:
                print(i,f,y)
        return res



if __name__ == '__main__':
    predictor=Predictor(
        model_path='model.pkl',
        classes_path='classes.txt'
    )
    predictor.test_dir(
        '/home/ars/disk/chaoyuan/dataset/汽车分类/raw',
        out_dir="/home/ars/disk/chaoyuan/dataset/汽车分类/raw_test_results",
        font_path='arsocr/utils/msyh.ttf'
    )





import torch
print("PyTorch Version: ",torch.__version__)
import torch.nn as nn
import torch.optim as optim
import numpy as np
import torchvision
from torchvision import datasets, models, transforms
import matplotlib.pyplot as plt
import time
import os,glob,shutil
import copy
# print("PyTorch Version: ",torch.__version__)
# print("Torchvision Version: ",torchvision.__version__)
from PIL import Image
from wpkit.utils import  remake
# data_dir = "/home/ars/disk/chaoyuan/dataset/汽车分类/颜色/car_color_dataset/val"
data_dir = "/home/ars/sda5/data/chaoyuan/datasets/classify_datasets/申请表字段有无签名/val"
# data_dir = "/home/ars/sda5/data/chaoyuan/datasets/classify_datasets/公章/val"
err_dir=data_dir+'_errors'
remake(err_dir)
classes=os.listdir(data_dir)
classes.sort()
imgs=[data_dir+'/'+i for i in os.listdir(data_dir)]
# model_path='trained_models/车型识别/model.pkl'
model_path='model.pkl'
###########################
model_name='resnet'
if model_name=='resnet':
    # input_size=(224,224)
    input_size=(150,400)
data_transforms = {
    'train': transforms.Compose([
        transforms.RandomResizedCrop(input_size),
        transforms.RandomHorizontalFlip(),
        transforms.ToTensor(),
        transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])
    ]),
    'val': transforms.Compose([
        transforms.Resize(input_size),
        # transforms.CenterCrop(input_size),
        transforms.ToTensor(),
        transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])
    ]),
}
transform=data_transforms['val']
device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")

###########################
model=torch.load(model_path)
model.to(device)
model.eval()

###########################
os.system('killall display')


def iter_data():
    for cls in os.listdir(data_dir):
        cls_dir = data_dir + '/' + cls
        for f in glob.glob(cls_dir+'/*.jpg'):
            yield f,cls

correct=0
total=0
for i,(f,cls) in enumerate(iter_data()):
    im = Image.open(f)
    im2=im
    im = transform(im).float()
    im = torch.tensor(im, requires_grad=False)
    im = im.unsqueeze(0)
    im=im.to(device)
    y = model(im)
    y = torch.argmax(y).cpu().int()
    y=int(y)
    # print(y,len(classes))
    pred=classes[y]
    if pred==cls:
        correct+=1
    else:
        f2=err_dir+'/%s'%(pred)+os.path.basename(f)
        shutil.copy(f,f2)
    total+=1
    # print(cls==pred)
    print(i,f,y,pred,cls)

accuracy=correct/total
print('total: %s , accuracy : %s'%(total,accuracy))




import cv2
from PIL import Image,ImageDraw,ImageFont
import numpy as np
from matplotlib import pyplot as plt
import os,shutil,glob

#########################Fonts###########################
default_font=None
default_font_path=None
def get_default_font(font_size=None):
    return ImageFont.truetype(default_font_path,font_size)
def set_font_path(path):
    global default_font_path
    default_font_path=path
def set_font(path,size=32):
    global default_font
    default_font=ImageFont.truetype(path,size=size)
def test_font_dir(dir,dst=None,text=None):
    fs=glob.glob(dir+'/*.ttf')+glob.glob(dir+'/*.ttc')+glob.glob(dir+'/*.otf')
    dst=dst or dir+'/test_font'
    for i,f in enumerate(fs):
        test_font(f,dst,text=text)
        print(i,f)
    print('finished.')
def test_font(path,dst='./test_font',text=None):
    img_dir=dst+'/imgs'
    log_file=dst+'/font_errors.txt'
    bad_fonts=dst+'/bad_fonts.txt'
    img=blank_rgb(size=(1024,32))
    font=ImageFont.truetype(path,size=24)
    text=text or 'Hello! 今天过得怎么样,~!#$%^&*()_+=-'
    try:
        img=draw_text(img,text=text,font=font)
    except:
        msg='Error occured when handle %s'%(path)
        print(msg)
        with open(log_file,'a',encoding='utf-8') as f:
            f.write(msg+'\n')
        with open(bad_fonts,'a',encoding='utf-8') as f:
            f.write(path+'\n')
        return
    if not os.path.exists(img_dir):
        os.makedirs(img_dir)
    img.save(img_dir+'/'+os.path.basename(path)[:-3]+'.jpg')
####################Generate#####################
def blank_rgb(size=(512,48),color='white'):
    img=Image.new('RGB',size,color)
    return img
def new_blank_img_as(img):
    img=Image.new('RGB',size=img.size,color='white')
    return img
###################Size#######################
def resize_to_fixed_height(img,height):
    w,h=img.size
    r=h/height
    nw=int(w/r)
    nh=int(h/r)
    img=img.resize((nw,nh))
    return img
def resize_to_fixed_width(img,width):
    w,h=img.size
    r=w/width
    nw=int(w/r)
    nh=int(h/r)
    img=img.resize((nw,nh))
    return img
def resize_by_scale(img,scale):
    w, h = img.size
    r = scale
    nw = int(w * r)
    nh = int(h * r)
    img = img.resize((nw, nh))
    return img
def limit_size(img,limits):
    w,h=img.size
    mw,mh=limits
    rw=w/mw
    rh=h/mh
    r=max(rw,rh)
    if r<=1:
        return img
    nw=int(w/r)
    nh=int(h/r)
    img=img.resize((nw,nh))
    return img
#####################Draw#########################
def concat_imgs_horizontal(imgs):
    imgs=[cv2img(img) for img in imgs]
    img=np.concatenate(imgs,axis=1)
    return pilimg(img)
def concat_imgs_vertical(imgs):
    imgs=[cv2img(img) for img in imgs]
    img=np.concatenate(imgs,axis=0)
    return pilimg(img)
def draw_boxes_with_label(img,boxes,offset=(0,-16),box_color='red',text_color='green',line_width=5):
    ofx,ofy=offset
    for box,label in boxes:
        l,t,r,b=box
        img=draw_box(img,box,outline=box_color,width=line_width)
        img=draw_text(img,text=label,xy=(l+ofx,t+ofy),fill=text_color)
    return img
def mark_img(img,text,font_path=None,font_size=None):
    imw,imh=img.size
    font_path=font_path or default_font_path
    if font_path:
        font_size=font_size or min(32,imw//len(text))
        font=ImageFont.truetype(font_path,size=font_size)
    else:
        font=None
    def calc_pos(imsize,boxsize):
        imw,imh=imsize
        w,h=boxsize
        x=imw//2-w//2
        y=imh//2-h//2
        return (x,y)

    xy=calc_pos(img.size,(font.size*len(text),font.size))
    img=draw_text(img,text,xy=xy,fill='red',font=font)
    return img
def draw_text(img,text,xy=(0,0),fill='black',font=None):
    font=font or default_font
    draw=ImageDraw.ImageDraw(img)
    draw.text(xy,text=text,fill=fill,font=font)
    return img
def draw_boxes(img,boxes,copy=False,*args,**kwargs):
    if copy:
        img=img.copy()
    for box in boxes:img=draw_box(img,box,copy=False,*args,**kwargs)
    return img
def draw_box(img,box,copy=True,width=5,outline='red',fill=None):
    box=tuple(box)
    if copy:
        img=img.copy()
    draw=ImageDraw.Draw(img)
    draw.rectangle((box[:2],box[2:]),width=width,outline=outline,fill=fill)
    return img
def draw_polygon(img,points,color='blue',width=1):
    points=[tuple(list(p)) for p in points]
    draw=ImageDraw.Draw(img)
    points.append(points[0])
    draw.line(points,fill=color,width=width)
    return img
def draw_textboxes(img,textboxes,copy=False,font_size=32):
    if copy:
        img=img.copy()
    for textbox in textboxes:
        img=draw_textbox(img,textbox,copy=False,font_size=font_size)
    return img
def draw_textbox(img,textbox,copy=True,font_size=None):
    import os
    font_size=font_size or 32
    box,text=textbox
    if copy:
        img=img.copy()
    draw = ImageDraw.Draw(img)
    font=ImageFont.truetype(os.path.dirname(__file__)+'/msyh.ttf',size=font_size)
    draw.text(box[:2],text=text,fill='black',font=font)
    return img
def pad_text_right(img,text,pad_width=400,pad_ratio=1,font=None,font_size=None,line_length=None):
    # img=cv2img(img)
    # h,w=img.shape[:2]
    # blank=np.zeros_like(img)
    def get_max_line_width(text):
        lines=text.split('\n')
        lens=[len(line) for line in lines]+[20]
        return max(*lens)
    line_length=line_length or max(get_max_line_width(text),20)
    w,h=img.size
    if pad_width:
        pad_width=int(w*pad_ratio)
    bw=pad_width
    canvas = Image.new(img.mode, (w+bw,h),'white')
    blank=Image.new(img.mode,(bw,h),color='white')
    font_size=font_size or max(16,int(bw/line_length))
    default_font = get_default_font(font_size=font_size)
    font = font or default_font
    blank=draw_text(blank,text,(5,5),font=font)
    canvas.paste(img)
    canvas.paste(blank,(w,0,w+bw,h))
    return canvas
########################Crop########################
def crop_boxes(img,boxes):
    imgs=[]
    for box in boxes:
        im=crop(img,box)
        imgs.append(im)
    return imgs
def iter_boxes(img,boxes,do):
    reses=[]
    for box in boxes:
        im=crop(img,box)
        res=do(im)
        reses.append(res)
    return reses
def crop(img,bbox):
    return img.crop(bbox)
def crop_by_ratio(img,rbox):
    w,h=img.size
    box=tuple([int(x) for x in (rbox[0]*w,rbox[1]*h,rbox[2]*w,rbox[3]*h)])
    return img.crop(box)
def crop_quad(img,box):
    p0, p1, p2, p3 = box
    (x0, y0), (x1, y1), (x2, y2), (x3, y3) = box
    w,h=((x1-x0+x2-x3)//2,(y3-y0+y2-y1)//2)
    w,h=int(w),int(h)
    M=cv2.getPerspectiveTransform(np.float32([p0,p1,p3,p2]),np.float32([[0,0],[w,0],[0,h],[w,h]]))
    img=cv2.warpPerspective(img,M,(w,h))
    return img
def crop_quads(img,boxes):
    bims=[]
    for box in boxes:
        bims.append(crop_quad(img,box))
    return bims
#######################Show#############################
def cv2img(img):
    if isinstance(img,Image.Image):
        img=np.array(img)
        if len(img.shape)==3:img=img[:,:,::-1]
        return img
    return img
def pilimg(img):
    if isinstance(img,Image.Image):return img
    if isinstance(img,np.ndarray):
        if len(img.shape)==3:img=img[:,:,::-1]
    return Image.fromarray(np.array(img).astype(np.uint8))
def pilimshow(x,*args,**kwargs):
    x=pilimg(x)
    x.show(*args,**kwargs)
def cv2imshow(x,title='cv2 image'):
    x=cv2img(x)
    cv2.imshow(title,x)
    cv2.waitKey(0)
def pltimshow(x,*args,**kwargs):
    x=cv2img(x)
    plt.imshow(x,*args,**kwargs)
    plt.show()
########################ImageSaver###########################
class ImageSaver:
    def __init__(self,save_dir=None,remake_dir=False,start_file_index=0):
        save_dir=save_dir or './tmp_image_save_dir'
        import time
        if os.path.exists(save_dir) and remake_dir:
            shutil.rmtree(save_dir)
            time.sleep(0.01)
        if not os.path.exists(save_dir):
            os.makedirs(save_dir)
        self.save_dir=save_dir
        self.remake_dir=remake_dir
        self.file_index=start_file_index
        self.alive=True
    def deactive(self):
        self.alive=False
    def active(self):
        self.alive=True
    def save(self,img,name=None):
        if not self.alive:
            return False
        name=name or '%s.jpg'%self.file_index
        name=os.path.join(self.save_dir,name)
        if isinstance(img, Image.Image):
            img.save(name)
        else:
            cv2.imencode('.jpg', img)[1].tofile(name)
        self.file_index+=1
        return name



import numpy as np
import math

def bounding_rect(points):
    points=np.array(points)
    l=np.min(points[:,0])
    r=np.max(points[:,0])
    t=np.min(points[:,1])
    b=np.max(points[:,1])
    return np.array([l,t,r,b])
def scale(points,scale):
    if isinstance(scale,(tuple,list)):
        scaleX,scaleY=scale
    else:
        scaleX=scaleY=scale
    points=np.array(points)*np.array([scaleX,scaleY])
    points=points.astype(np.int)
    return points
def hflip(points,width):
    points=np.array(points)
    points[:,0]=width-points[:,0]
    return points
def vflip(points,height):
    points=np.array(points)
    points[:,1]=height-points[:,1]
    return points
def translate(points,offset):
    points=np.array(points)+np.array(offset)
    return points
def get_translate_range(points,limits):
    points=np.array(points).reshape(-1,2)
    if len(limits)==2:
        ml=mt=0
        mr,mb=limits
    else:
        ml,mt,mr,mb=limits
    l,t,r,b=bounding_rect(points)
    # xrange,yrange=[ml-l,mr-r],[mt-t,mb-b]
    range=[ml-l,mt-t,mr-r,mb-b]
    return range
    # return xrange,yrange

def rotate(points,degree,center,img_size=None,expand=None):
    angle=math.radians(degree)
    M=[
        [math.cos(angle),math.sin(angle)],
        [-math.sin(angle),math.cos(angle)]
    ]
    M=np.array(M)
    points=np.array(points)
    center=np.mat(center)
    def _rotate_points(points,M,center):
        points=M.dot((points-center).T)+center.T
        points=np.array(points.T)
        return points
    points=_rotate_points(points,M,center)
    if expand:
        def _get_img_boundary(img_size):
            w,h=img_size
            bnd=np.array([[0,0],[w,0],[0,h],[w,h]])
            bnd=_rotate_points(bnd,M,center)
            bnd=bounding_rect(bnd)
            return bnd
        l,t,r,b=_get_img_boundary(img_size)
        points=points-np.array([l,t])
    points=np.array(points).astype(np.int).reshape((-1,2))
    return points

def shear_x(points,degree,img_size=None,expand=None):
    '''[x2,y2].T=[[1,sin],[0,cos]]*[x1,y1].T

    '''
    angle=math.radians(degree)
    M=np.array([
        [1,math.sin(angle)],
        [0,1]
    ])
    points=np.mat(points)*M.T
    if expand:
        def _get_img_boundary(img_size):
            w,h=img_size
            bnd=np.mat([[0,0],[w,0],[0,h],[w,h]])
            bnd=bnd*M.T
            bnd=bounding_rect(bnd)
            return bnd

        l,t,r,b=_get_img_boundary(img_size)
        points = points - np.array([l, t])
    points = np.array(points).astype(np.int).reshape((-1, 2))
    return points

def shear_y(points,degree,img_size=None,expand=None):
    '''[x2,y2].T=[[1,sin],[0,cos]]*[x1,y1].T

    '''
    angle=math.radians(degree)
    M=np.array([
        [1,0],
        [-math.sin(angle),1]
    ])
    points=np.mat(points)*M.T
    # expand=None
    if expand:
        def _get_img_boundary(img_size):
            w,h=img_size
            bnd=np.mat([[0,0],[w,0],[0,h],[w,h]])
            bnd=bnd*M.T
            bnd=bounding_rect(bnd)
            return bnd

        l,t,r,b=_get_img_boundary(img_size)
        points = points - np.array([l, t])
    points = np.array(points).astype(np.int).reshape((-1, 2))
    return points




# def
# def shear():
#     x2-x1=y1*sin
#     y2=y1*cos
#     [x2,y2]=[x1,y1]*[[1,sin].T,[0,cos].T]
#     [x2,y2].T=[[1,sin],[0,cos]]*[x1,y1].T
# def rotate():
#     [x2,y2]-[c2x,c2y]=[[cos,-sin],[sin,cos]]*([x1,y1]-[c1x,c1y])
#     [x2,y2]=M*[x1,y1]-M*[c1x,c1y]+[c2x,c2y]


# def shift(points,offset):



import os, glob
import cv2
import numpy as np
from PIL import Image
# from arsocr.utils.extra.rectify import rectify,detect_angle,rotate
# from arsocr.utils.allutils import crop_quad,organize_points
# from wpkit.cv.ocr.utils import crop_quad,organize_points
# from wpkit.cv.table.rectify import rectify,detect_angle,rotate
# from wpkit.cv import utils as cvutils
# from wpkit.cv.utils import ImageSaver
# from wpkit.pjtools import Timer

# imsaver=ImageSaver(save_dir=r'/home/ars/disk/work/超远/chaoyuan-part1-dev/data/results/tmp',remake_dir=True)
# imsaver.deactive()
# def save_to_dir(img,name=None):
#     return imsaver.save(img,name)

from wk.cv import utils as cvutils

def get_table_contour(img,exclude_paper_edge_thresh=20):
    n=4
    # src = img.copy()

    img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)

    img = cv2.GaussianBlur(img, (3, 3), 0)

    img_blur=img
    img = cv2.bitwise_not(img)

    img_inverse=img
    # save_to_dir(img)
    AdaptiveThreshold = cv2.adaptiveThreshold(img, 255, cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY, 15, -2)

    # mask=AdaptiveThreshold
    mask = AdaptiveThreshold
    # save_to_dir(mask)

    # contours, hierarchy = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
    contours, hierarchy = cv2.findContours(mask, cv2.RETR_CCOMP, cv2.CHAIN_APPROX_SIMPLE)
    # contours, hierarchy = cv2.findContours(mask, cv2.RETR_LIST, cv2.CHAIN_APPROX_SIMPLE)
    # contours, hierarchy = cv2.findContours(mask, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)

    contours = sorted(contours, key=cv2.contourArea, reverse=True)
    contours = list(filter(lambda cont: cv2.contourArea(cont) > 30, contours))
    num = min(n, len(contours))
    contours = contours[:num]

    # 上面得到的contour可能是单个轮廓都是不完整的，但是叠加在一起有完整的，因此从第一次的contours  image中重新findContour
    blank=np.zeros_like(img_blur)
    img_contours=cv2.drawContours(blank,contours,-1,255,3)

    # save_to_dir(img_contours)
    contours, hierarchy = cv2.findContours(img_contours, cv2.RETR_CCOMP, cv2.CHAIN_APPROX_SIMPLE)
    contours = sorted(contours, key=cv2.contourArea, reverse=True)
    contours = list(filter(lambda cont: cv2.contourArea(cont) > 30, contours))
    num = min(n, len(contours))
    contours = contours[:num]


    ones=np.ones_like(img_blur)
    thresh=exclude_paper_edge_thresh
    # return contours[0]
    for i in range(len(contours)):
        mask=cv2.fillPoly(ones.copy(),[contours[i]],0)
        im=img_blur*mask
        mean=im.mean()
        if mean>thresh:
            return contours[i]
    if len(contours):
        return contours[0]
    return None
def get_table(img,max_approx_poly_distance=200,exclude_paper_edge_thresh=20,scale=1,src_img=None):
    # T = Timer(mute=True)
    src_img = src_img or img.copy()

    if not scale==1:
        h,w=img.shape[:2]
        h,w=int(scale*h),int(scale*w)
        img=cv2.resize(img,(w,h))

    #
    contour=get_table_contour(img,exclude_paper_edge_thresh=exclude_paper_edge_thresh)

    if contour is None:
        return None
    # im=cv2.drawContours(img.copy(),[contour],-1,[255,0,0],5)
    # save_to_dir(im)
    approx=cv2.approxPolyDP(contour,max_approx_poly_distance,True)

    # print(approx.shape)

    if len(approx)==4:
        approx=[p[0] for p in approx]
        quad=cvutils.organize_quad_points(approx)
        quad=np.array(quad)/scale
        # table=crop_quad(src_img,box)
    else:
        rect=cv2.boundingRect(approx)
        rect=np.array(rect)/scale
        # print(rect)
        x1,y1,w,h=rect
        quad=[[x1,y1],[x1+w,y1],[x1+w,y1+h],[x1,y1+h]]
    return np.array(quad).astype(np.int)
        # table=src_img[y1:y1+h,x1:x1+w]

    # save_to_dir(table)
    # return table
def cv_imread(f):
    from wpkit.cv.utils import cv2img
    return cv2img(Image.open(f))
def demo():
    # from wpkit.pjtools import run_timer,Timer
    imsaver.deactive()
    # dir = '/home/ars/disk/chaoyuan/数据整理/ocr测试图片/exp/imgs'
    # dir = r'/home/ars/disk/chaoyuan/数据整理/ocr测试图片/安检报告'
    # dir = r'/home/ars/disk/chaoyuan/数据整理/ocr测试图片/申请表'
    # dir = r'/home/ars/disk/chaoyuan/数据整理/ocr测试图片/在用车检验'
    dir = r'/home/ars/disk/chaoyuan/数据整理/ocr测试图片/保险单'
    fs = glob.glob(dir + '/*.jpg')
    # fs=['']
    T= Timer()
    for i, f in enumerate(fs):
        assert os.path.exists(dir)
        assert os.path.exists(f)
        img = cv_imread(f)
        src_img=img.copy()

        scale=0.4
        img=cvutils.transform.rescale(img,scale)
        # img = rectify(img)
        # img=rectify(img)
        angle=detect_angle(img)
        if angle and abs(angle)>1:
            img=rotate(img,-angle)
            src_img=rotate(src_img,-angle)

        quad=get_table(img)
        quad=(quad/scale).astype(np.int)
        table=crop_quad(src_img,quad)
        save_to_dir(table)
        T.step()
    print(T.mean())
if __name__ == '__main__':

    # corner()
    demo()




import math
##########################Box Operations#############################
def resize_box(box,size):
    cx,cy,w,h=ltrb_to_ccwh(box)
    nw,nh=size
    l=cx-nw//2
    r=cx+nw//2
    t=cy-nh//2
    b=cy+nh//2
    return [l,t,r,b]
def rescale_box(box,scale):
    '''rescale with  image'''
    if isinstance(scale,(tuple,list)):
        scaleX,scaleY=scale
    else:
        scaleX=scaleY=scale
    l,t,r,b=box
    l*=scaleX
    r*=scaleX
    t*=scaleY
    b*=scaleY
    return [l,t,r,b]
def get_rotate_info(angle,cx,cy,h,w):
    import math
    import numpy as np
    angle = math.radians(angle)
    cos = math.cos(angle)
    sin = math.sin(angle)
    M = np.array([[cos, sin], [-sin, cos]])
    cos = abs(cos)
    sin = abs(sin)
    nw = w * cos + h * sin
    nh = w * sin + h * cos
    return M,nw,nh
def rotate_points(points,angle,cx,cy,h,w):
    import numpy as np
    M,nw,nh=get_rotate_info(angle,cx,cy,h,w)
    points=np.array(points)
    original_shape=points.shape
    points=points.reshape((-1,2))
    points=M.dot((points-np.array([cx,cy])).T).T+np.array([nw//2,nh//2])
    points=points.reshape(original_shape)
    return points

def rotate_boxes(boxes,angle,cx,cy,h,w):
    '''原创'''
    boxes=[list(box) for box in boxes]
    corners = [ltrb_to_four_corners(box) for box in boxes]
    corners=rotate_points(corners,angle,cx,cy,h,w)
    boxes=[four_coners_to_ltrb(quad_to_four_corners(organize_quad_points(four_corners_to_quad(list(box))))) for box in corners]
    return boxes
def rotate_box(box, angle, cx, cy, h, w):
    return rotate_boxes([box],angle,cx,cy,h,w)
def shift_box(box,offset,limits=None):
    l,t,r,b=box
    ofx,ofy=offset
    l+=ofx
    r+=ofx
    t+=ofy
    b+=ofy
    return limit_box([l,t,r,b],limits=limits)
def translate_box(box,offset,limits=None):
    return shift_box(box,offset,limits)
def flip_box_horizontal(box,imsize):
    imw,imh=imsize
    l,t,r,b=box
    l=imw-r
    r=imw-l
    return [l,t,r,b]
def flip_box_vertical(box,imsize):
    imw,imh=imsize
    l,t,r,b=box
    t=imh-b
    b=imh-t
    return [l,t,r,b]
def pad_box(box,pad_size=5,pad_ratio=None,limits=None):
    from math import inf
    limits=limits or (-inf,-inf,inf,inf)
    l,t,r,b=box
    bh=b-t
    bw=r-l
    if pad_ratio is not None:
        if isinstance(pad_ratio,(tuple,list)):
            if len(pad_ratio)==2:
                pad_l=pad_r=int(min(bh,bw)*pad_ratio[0])
                pad_t=pad_b=int(min(bh,bw)*pad_ratio[1])
            else:
                assert len(pad_ratio)==4
                pad_l= int(min(bh, bw) * pad_ratio[0])
                pad_t= int(min(bh, bw) * pad_ratio[1])
                pad_r= int(min(bh, bw) * pad_ratio[2])
                pad_b= int(min(bh, bw) * pad_ratio[3])
        else:
            pad_l=pad_t=pad_r=pad_b=int(min(bh,bw)*pad_ratio)
    else:
        if isinstance(pad_size,(tuple,list)):
            if len(pad_size)==2:
                pad_l=pad_r=pad_size[0]
                pad_t=pad_b=pad_size[1]
            else:
                assert len(pad_size)==4
                pad_l,pad_t,pad_r,pad_b=pad_size
        else:
            pad_l=pad_t=pad_r=pad_b=pad_size
    l-=pad_l
    t-=pad_t
    r+=pad_r
    b+=pad_b
    return limit_box([l,t,r,b],limits=limits)
def limit_box(box,limits=None):
    if limits is None:return box
    if len(limits)==2:
        ml,mt=0,0
        mr,mb=limits
    else:
        assert len(limits)==4
        ml,mt,mr,mb=limits
    l,t,r,b=box
    l=max(ml,l)
    t=max(mt,t)
    r=min(mr,r)
    b=min(mb,b)
    if l>=r:
        return None
    if t>=b:return None
    return [l,t,r,b]
############################Sort Operations#############################
def min_enclosing_box(boxes):
    assert len(boxes)
    ml,mt,mr,mb=boxes[0]
    for box in boxes[1:]:
        l,t,r,b=box
        ml=min(ml,l)
        mt=min(mt,t)
        mr=max(mr,r)
        mb=max(mb,b)
    return [ml,mt,mr,mb]
def box_in_area(box,size):
    l,t,r,b=box
    mr,mb=size
    if l<0 or t<0 or r>mr or b>mb:
        return False
    else:
        return True

def organize_quad_points(box):
    '''
    turn into clock-wise points: quad
    p0:lt,p1:rt,p2:rb,p4:lb
    '''
    p0,p3, p1,p2 = sorted(box, key=lambda p: p[0])
    p03=[p0,p3]
    p12=[p1,p2]
    p0, p3 = sorted(p03, key=lambda p: p[1])
    p1, p2 = sorted(p12, key=lambda p: p[1])
    return [p0,p1,p2,p3]
def sort_boxes(boxes):
    from functools import cmp_to_key
    boxes=[ltrb_to_ccwh(box) for box in boxes]
    boxes=[[box] for box in boxes]
    def block_cmp(b1, b2):
        list1 = [b1[0][0], b1[0][1], b1[0][2], b1[0][3]]
        list2 = [b2[0][0], b2[0][1], b2[0][2], b2[0][3]]
        if len(b1[0]) == 4:
            list1[0] += list1[2] / 2
            list1[1] += list1[3] / 2
            list2[0] += list2[2] / 2
            list2[1] += list2[3] / 2

        flag = 1
        if list1[0] > list2[0]:
            list1, list2 = list2, list1
            flag = -1

        if list2[1] + list1[3] / 2 < list1[1]:
            return flag
        return -flag

    boxes.sort(key=cmp_to_key(block_cmp), reverse=False)
    boxes=[box[0] for box in boxes]
    boxes=[ccwh_to_ltrb(box) for box in boxes]
    return boxes
############################ConvertingTools#############################
def ltrb_to_four_corners(box):
    '''four corners: 8-d vector clock-wise'''
    l,t,r,b=box
    box2=[l,t,r,t,r,b,l,b]
    return box2
def four_coners_to_ltrb(box,sort=False):
    x1,y1,x2,y2,x4,y4,x3,y3=box
    l=min(x1,x3)
    r=max(x2,x4)
    t=min(y1,y2)
    b=max(y3,y4)
    if sort:
        newbox = [min(l, r), min(t, b), max(l, r), max(t, b)]
    else:
        newbox=[l,t,r,b]
    return newbox
def ltrb_to_ccwh(box):
    l, t, r, b = box
    w = r - l
    h = b - t
    cx = (l + r) // 2
    cy = (t + b) // 2
    return [cx,cy,w,h]
def ccwh_to_ltrb(box):
    cx,cy,w,h=box
    l=cx-w//2
    t=cy-h//2
    r=cx+w//2
    b=cy+h//2
    return [l,t,r,b]
def oowh_to_ltrb(box):
    '''o:origin'''
    x, y, w, h = box
    return [int(i) for i in (x, y, x + w, y + h)]
def ltrb_to_oowh(box):
    l, t, r, b = box
    return [l, t, r - l, b - t]
def xywh_to_ltrb(box):
    '''same with oowh'''
    x,y,w,h=box
    return [int(i) for i in (x,y,x+w,y+h)]
def ltrb_to_xywh(box):
    l,t,r,b=box
    return [l,t,r-l,b-t]
def quad_to_ltrb(box):
    '''quad: clock-wise four points'''
    (x0,y0),(x1,y1),(x2,y2),(x3,y3)=box
    return [min(x0,x3),min(y0,y1),max(x1,x2),max(y2,y3)]
def ltrb_to_quad(box):
    x0,y0,x1,y1=box
    return [(x0,y0),(x1,y0),(x1,y1),(x0,y1)]
def quad_to_four_corners(quad):
    (x0, y0), (x1, y1), (x2, y2), (x3, y3) = quad
    return [x0,y0,x1,y1,x2,y2,x3,y3]
def four_corners_to_quad(box):
    x0,y0,x1,y1,x2,y2,x3,y3 = box
    return [(x0, y0), (x1, y1), (x2, y2), (x3, y3)]
###########################Calculations##############################
def calc_iou(box1,box2):
    l1,t1,r1,b1=box1
    l2,t2,r2,b2=box2
    w1,h1=r1-l1,b1-t1
    w2,h2=r2-l2,b2-t2
    width=min(r1,r2)-max(l1,l2)
    height=min(b1,b2)-max(t1,t2)
    width=max(width,0)
    height=max(height,0)
    area=width*height
    return area/(w1*h1+w2*h2-area)
def calc_iou2(box1,box2):
    width=min(box1[2],box2[2])-max(box1[0],box2[0])
    height=min(box1[3],box2[3])-max(box1[1],box2[1])
    width=max(width,0)
    height=max(height,0)
    area=width*height
    return area/((box1[2]-box1[0])*(box1[3]-box1[1])+(box2[2]-box2[0])*(box2[3]-box2[1])-area)
def calc_iou_batch_with_batch(boxes1,boxes2):
    box1=boxes1
    box2=boxes2
    import numpy as np
    box1=np.array(box1)
    box2=np.array(box2)
    width = np.min(np.vstack([box1[...,2], box2[...,2]]),axis=0) - np.max(np.vstack([box1[...,0], box2[...,0]]),axis=0)
    height = np.min(np.vstack([box1[...,3], box2[...,3]]),axis=0) - np.max(np.vstack([box1[...,1], box2[...,1]]),axis=0)
    width = np.clip(width,0,np.inf)
    height = np.clip(height,0,np.inf)
    area = width * height
    return area / ((box1[...,2] - box1[...,0]) * (box1[...,3] - box1[...,1]) + (box2[...,2] - box2[...,0]) * (box2[...,3] - box2[...,1]) - area)
def calc_iou_batch(boxes,box):
    '''calculate iou between batch bboxes with one bbox , 原创'''
    box1=boxes
    box2=box
    import numpy as np
    box1 = np.array(box1)
    box2 = np.array(box2)
    shape=box1.shape
    width = np.min(np.vstack([box1[..., 2], np.full(shape[:-1],box2[2])]), axis=0) - np.max(np.vstack([box1[..., 0], np.full(shape[:-1],box2[0])]),axis=0)
    height = np.min(np.vstack([box1[..., 3], np.full(shape[:-1],box2[3])]), axis=0) - np.max(np.vstack([box1[..., 1], np.full(shape[:-1],box2[1])]),axis=0)
    width = np.clip(width, 0, np.inf)
    height = np.clip(height, 0, np.inf)
    area = width * height
    return area / ((box1[...,2] - box1[...,0]) * (box1[...,3] - box1[...,1]) + (box2[2] - box2[0]) * (box2[3] - box2[1]) - area)
def calc_quad_angle(quad):
    '''cal angle of a quad'''
    p0,p1,p2,p3=quad
    def center(p1,p2):
        x1,y1=p1
        x2,y2=p2
        return (x1+x2)/2,(y1+y2)/2
    x1,y1=center(p0,p3)
    x2,y2=center(p1,p2)
    angle=math.atan2(y1-y2,x2-x1)*180/math.pi
    return angle
def calc_box_area(box):
    l,t,r,b=box
    return (r-l)*(b-t)
##############################TextBox################################
class TextBox(dict):
    def __init__(self):
        super().__init__()
    def parse(self):
        p0,p1,p2,p3=quad=self['quad']
        xmin,ymin,xmax,ymax=l,t,r,b=ltrb=quad_to_ltrb(quad)
        ox,oy,w,h=oowh=ltrb_to_oowh(ltrb)
        cx,cy,w,h=ccwh=ltrb_to_ccwh(ltrb)
        angle=calc_quad_angle(quad)
        text=None
        prob=None
        self.update(
            ltrb=ltrb,oowh=oowh,ccwh=ccwh,
            xmin=xmin,ymin=ymin,xmax=xmax,ymax=ymax,
            l=l,t=t,r=r,b=b,
            ox=ox,oy=oy,w=w,h=h,
            cx=cx,cy=cy,
            angle=angle,
            text=text,
            prob=prob
        )
        return self



import numpy as np
import random
import numbers
import cv2
from PIL import Image
import wpcv
from wpcv.utils.transforms import pil as IM
from wpcv.utils.transforms import points as PT
from wpcv.utils.augmentations.base import Compose, Zip, ToPILImage
import wpcv.utils.augmentations.base as BT


class BboxesToPoints(object):
    def __call__(self, img, bboxes):
        points = np.array(bboxes).reshape((-1, 2, 2))
        return img, points


class PointsToBboxes(object):
    def __call__(self, img, points):
        bboxes = np.array(points).reshape((-1,4))
        return img, bboxes


class Reshape(object):
    def __init__(self, shape):
        self.target_shape = shape

    def __call__(self, x):
        return np.array(x).reshape(self.target_shape)


class Identical(object):
    def __call__(self, *args):
        if len(args)==1:
            return args[0]
        return args


class Scale(object):
    def __init__(self, scales):
        if isinstance(scales, (tuple, list)):
            scaleX, scaleY = scales
        else:
            scaleX = scaleY = scales
        self.scaleX, self.scaleY = scaleX, scaleY

    def __call__(self, img, points):
        scaleX, scaleY = self.scaleX, self.scaleY
        img = IM.scale(img, (scaleX, scaleY))
        points = PT.scale(points, (scaleX, scaleY))
        return img, points


class Resize(object):
    def __init__(self, size):
        self.size = size

    def __call__(self, img, points):
        w, h = img.size
        nw, nh = self.size
        scaleX, scaleY = nw / w, nh / h
        img = IM.scale(img, (scaleX, scaleY))
        points = PT.scale(points, (scaleX, scaleY))
        return img, points


class RandomHorizontalFlip(object):
    def __init__(self, p=0.5):
        self.p = p

    def __call__(self, img, points):
        imw, imh = img.size
        if random.random() < self.p:
            img = IM.hflip(img)
            points = [PT.hflip(pnts, imw) for pnts in points]
        return img, points

    def __repr__(self):
        return self.__class__.__name__ + '(p={})'.format(self.p)


class RandomVerticalFlip(object):
    def __init__(self, p=0.5):
        self.p = p

    def __call__(self, img, points):
        imw, imh = img.size
        if random.random() < self.p:
            img = IM.vflip(img)
            points = [PT.vflip(pnts, imh) for pnts in points]
        return img, points

    def __repr__(self):
        return self.__class__.__name__ + '(p={})'.format(self.p)


class RandomTranslate(object):
    def __init__(self,max_offset=None,fillcolor='black'):
        if max_offset is not None and len(max_offset)==2:
            mx,my=max_offset
            max_offset=[-mx,-my,mx,my]
        self.max_offset=max_offset
        self.fillcolor=fillcolor
    def __call__(self, img,points):
        rang=PT.get_translate_range(points,img.size)
        if self.max_offset:
            def limit_box(box, limits=None):
                if limits is None: return box
                if len(limits) == 2:
                    ml, mt = 0, 0
                    mr, mb = limits
                else:
                    assert len(limits) == 4
                    ml, mt, mr, mb = limits
                l, t, r, b = box
                l = max(ml, l)
                t = max(mt, t)
                r = min(mr, r)
                b = min(mb, b)
                if l > r:
                    return None
                if t > b: return None
                return [l, t, r, b]
            rang=limit_box(rang,self.max_offset)
            if rang is None:
                return img,points
        ofx=random.randint(rang[0],rang[2])
        ofy=random.randint(rang[1],rang[3])
        img=IM.translate(img,offset=(ofx,ofy),fillcolor=self.fillcolor)
        points=[PT.translate(pnts,(ofx,ofy)) for pnts in points]
        return img,points

class RandomRotate(object):
    def __init__(self,degree,expand=True):
        self.degree=degree if not isinstance(degree,numbers.Number) else [-degree,degree]
        self.expand=expand
    def __call__(self,img,points):
        degree=random.random()*(self.degree[1]-self.degree[0])+self.degree[0]
        w, h = img.size
        img=IM.rotate(img,degree,expand=self.expand)
        points=[PT.rotate(pnts,degree,(w//2,h//2),img_size=(w,h),expand=self.expand) for pnts in points]
        return img,points

class RandomShearX(object):
    def __init__(self,degree):
        self.degree=degree if not isinstance(degree,numbers.Number) else [-degree,degree]
    def __call__(self,img,points):
        degree = random.random() * (self.degree[1] - self.degree[0]) + self.degree[0]
        w, h = img.size
        img = IM.shear_x(img, degree)
        points = [PT.shear_x(pnts, degree, img_size=(w, h), expand=True) for pnts in points]
        return img, points
class RandomShearY(object):
    def __init__(self,degree):
        self.degree=degree if not isinstance(degree,numbers.Number) else [-degree,degree]
    def __call__(self,img,points):
        degree = random.random() * (self.degree[1] - self.degree[0]) + self.degree[0]
        w, h = img.size
        img = IM.shear_y(img, degree)
        points = [PT.shear_y(pnts, degree, img_size=(w, h), expand=True) for pnts in points]
        return img, points
class RandomShear(object):
    def __init__(self,xdegree,ydegree=None):
        def get_param(param,defualt=None):
            if param is None:return defualt
            return param if not isinstance(param,numbers.Number) else [-param,param]
        self.xdegree=get_param(xdegree)
        self.ydegree=get_param(ydegree)


    def __call__(self,img,points):
        if self.xdegree:
            degree = random.random() * (self.xdegree[1] - self.xdegree[0]) + self.xdegree[0]
            w, h = img.size
            img = IM.shear_x(img, degree)
            points = [PT.shear_x(pnts, degree, img_size=(w, h), expand=True) for pnts in points]
        if self.ydegree:
            degree = random.random() * (self.ydegree[1] - self.ydegree[0]) + self.ydegree[0]
            w, h = img.size
            img = IM.shear_y(img, degree)
            points = [PT.shear_y(pnts, degree, img_size=(w, h), expand=True) for pnts in points]
        return img, points

class RandomAffine(object):
    """Random affine transformation of the image keeping center invariant

    Args:
        degrees (sequence or float or int): Range of degrees to select from.
            If degrees is a number instead of sequence like (min, max), the range of degrees
            will be (-degrees, +degrees). Set to 0 to deactivate rotations.
        translate (tuple, optional): tuple of maximum absolute fraction for horizontal
            and vertical translations. For example translate=(a, b), then horizontal shift
            is randomly sampled in the range -img_width * a < dx < img_width * a and vertical shift is
            randomly sampled in the range -img_height * b < dy < img_height * b. Will not translate by default.
        scale (tuple, optional): scaling factor interval, e.g (a, b), then scale is
            randomly sampled from the range a <= scale <= b. Will keep original scale by default.
        shear (sequence or float or int, optional): Range of degrees to select from.
            If shear is a number, a shear parallel to the x axis in the range (-shear, +shear)
            will be apllied. Else if shear is a tuple or list of 2 values a shear parallel to the x axis in the
            range (shear[0], shear[1]) will be applied. Else if shear is a tuple or list of 4 values,
            a x-axis shear in (shear[0], shear[1]) and y-axis shear in (shear[2], shear[3]) will be applied.
            Will not apply shear by default
        resample ({PIL.Image.NEAREST, PIL.Image.BILINEAR, PIL.Image.BICUBIC}, optional):
            An optional resampling filter. See `filters`_ for more information.
            If omitted, or if the image has mode "1" or "P", it is set to PIL.Image.NEAREST.
        fillcolor (tuple or int): Optional fill color (Tuple for RGB Image And int for grayscale) for the area
            outside the transform in the output image.(Pillow>=5.0.0)

    .. _filters: https://pillow.readthedocs.io/en/latest/handbook/concepts.html#filters

    """

    def __init__(self, degrees, translate=None, scale=None, shear=None, resample=False, fillcolor=0):
        if isinstance(degrees, numbers.Number):
            if degrees < 0:
                raise ValueError("If degrees is a single number, it must be positive.")
            self.degrees = (-degrees, degrees)
        else:
            assert isinstance(degrees, (tuple, list)) and len(degrees) == 2, \
                "degrees should be a list or tuple and it must be of length 2."
            self.degrees = degrees

        if translate is not None:
            assert isinstance(translate, (tuple, list)) and len(translate) == 2, \
                "translate should be a list or tuple and it must be of length 2."
            for t in translate:
                if not (0.0 <= t <= 1.0):
                    raise ValueError("translation values should be between 0 and 1")
        self.translate = translate

        if scale is not None:
            assert isinstance(scale, (tuple, list)) and len(scale) == 2, \
                "scale should be a list or tuple and it must be of length 2."
            for s in scale:
                if s <= 0:
                    raise ValueError("scale values should be positive")
        self.scale = scale

        if shear is not None:
            if isinstance(shear, numbers.Number):
                if shear < 0:
                    raise ValueError("If shear is a single number, it must be positive.")
                self.shear = (-shear, shear)
            else:
                assert isinstance(shear, (tuple, list)) and \
                    (len(shear) == 2 or len(shear) == 4), \
                    "shear should be a list or tuple and it must be of length 2 or 4."
                # X-Axis shear with [min, max]
                if len(shear) == 2:
                    self.shear = [shear[0], shear[1], 0., 0.]
                elif len(shear) == 4:
                    self.shear = [s for s in shear]
        else:
            self.shear = shear

        self.resample = resample
        self.fillcolor = fillcolor

    @staticmethod
    def get_params(degrees, translate, scale_ranges, shears, img_size):
        """Get parameters for affine transformation

        Returns:
            sequence: params to be passed to the affine transformation
        """
        angle = random.uniform(degrees[0], degrees[1])
        if translate is not None:
            max_dx = translate[0] * img_size[0]
            max_dy = translate[1] * img_size[1]
            translations = (np.round(random.uniform(-max_dx, max_dx)),
                            np.round(random.uniform(-max_dy, max_dy)))
        else:
            translations = (0, 0)

        if scale_ranges is not None:
            scale = random.uniform(scale_ranges[0], scale_ranges[1])
        else:
            scale = 1.0

        if shears is not None:
            if len(shears) == 2:
                shear = [random.uniform(shears[0], shears[1]), 0.]
            elif len(shears) == 4:
                shear = [random.uniform(shears[0], shears[1]),
                         random.uniform(shears[2], shears[3])]
        else:
            shear = 0.0

        return angle, translations, scale, shear

    def __call__(self, img,points):
        """
            img (PIL Image): Image to be transformed.

        Returns:
            PIL Image: Affine transformed image.
        """
        ret = self.get_params(self.degrees, self.translate, self.scale, self.shear, img.size)
        img=BT.F.affine(img, *ret, resample=self.resample, fillcolor=self.fillcolor)
        return img,points

def demo():
    transform = Compose([
        # BboxToPoints(),
        Resize((512, 512)),
        Zip([
            Compose([
                BT.ColorJitter(brightness=0.5, contrast=0.5, saturation=0.5, hue=0.5)
            ]),
            Identical()
        ]),
        # RandomRotate(30),
        # RandomShear(30,30),
        # RandomTranslate(max_offset=[100,100]),
        # RandomHorizontalFlip(),
        # RandomVerticalFlip(),
        # PointsToBbox()
    ])
    img = Image.open('/home/ars/图片/2019011816055827.jpg')

    points=[[
          255.7142857142857,
          243.1428571428571
        ],
        [
          305.7142857142857,
          243.1428571428571
        ],
        [
          310.0,
          306.4761904761905
        ],
        [
          254.28571428571428,
          305.04761904761904
        ]]
    img, points = transform(img, [points])
    # print(points)
    img=wpcv.draw_polygon(img,points[0])
    img.show()
    # print(img.size, box)


if __name__ == '__main__':
    demo()



from PIL import Image
import numpy

# defining azimuth, elevation, and depth
ele = numpy.pi/2.2 # radians
azi = numpy.pi/4.  # radians
dep = 10.          # (0-100)

# get a B&W version of the image
img = Image.open('/home/ars/图片/2019011816055827.jpg').convert('L')
# get an array
a = numpy.asarray(img).astype('float')
# find the gradient
grad = numpy.gradient(a)
# (it is two arrays: grad_x and grad_y)
grad_x, grad_y = grad
# getting the unit incident ray
gd = numpy.cos(ele) # length of projection of ray on ground plane
dx = gd*numpy.cos(azi)
dy = gd*numpy.sin(azi)
dz = numpy.sin(ele)
# adjusting the gradient by the "depth" factor
# (I think this is how GIMP defines it)
grad_x = grad_x*dep/100.
grad_y = grad_y*dep/100.
# finding the unit normal vectors for the image
leng = numpy.sqrt(grad_x**2 + grad_y**2 + 1.)
uni_x = grad_x/leng
uni_y = grad_y/leng
uni_z = 1./leng
# take the dot product
a2 = 255*(dx*uni_x + dy*uni_y + dz*uni_z)
# avoid overflow
a2 = a2.clip(0,255)
# you must convert back to uint8 /before/ converting to an image
img2 = Image.fromarray(a2.astype('uint8'))
img2.save('daisy2.png')


from PIL import Image,ImageChops,ImageFilter,ImageOps,ImageEnhance
import numpy as np
import numbers,math

# Color Transforms
def _is_pil_image(img):
    return isinstance(img, Image.Image)
def rgb_to_hsv(rgb):
    import numpy as np
    # Translated from source of colorsys.rgb_to_hsv
    # r,g,b should be a numpy arrays with values between 0 and 255
    # rgb_to_hsv returns an array of floats between 0.0 and 1.0.
    rgb = rgb.astype('float')
    hsv = np.zeros_like(rgb)
    # in case an RGBA array was passed, just copy the A channel
    hsv[..., 3:] = rgb[..., 3:]
    r, g, b = rgb[..., 0], rgb[..., 1], rgb[..., 2]
    maxc = np.max(rgb[..., :3], axis=-1)
    minc = np.min(rgb[..., :3], axis=-1)
    hsv[..., 2] = maxc
    mask = maxc != minc
    hsv[mask, 1] = (maxc - minc)[mask] / maxc[mask]
    rc = np.zeros_like(r)
    gc = np.zeros_like(g)
    bc = np.zeros_like(b)
    rc[mask] = (maxc - r)[mask] / (maxc - minc)[mask]
    gc[mask] = (maxc - g)[mask] / (maxc - minc)[mask]
    bc[mask] = (maxc - b)[mask] / (maxc - minc)[mask]
    hsv[..., 0] = np.select(
        [r == maxc, g == maxc], [bc - gc, 2.0 + rc - bc], default=4.0 + gc - rc)
    hsv[..., 0] = (hsv[..., 0] / 6.0) % 1.0
    return hsv


def hsv_to_rgb(hsv):
    import numpy as np
    # Translated from source of colorsys.hsv_to_rgb
    # h,s should be a numpy arrays with values between 0.0 and 1.0
    # v should be a numpy array with values between 0.0 and 255.0
    # hsv_to_rgb returns an array of uints between 0 and 255.
    rgb = np.empty_like(hsv)
    rgb[..., 3:] = hsv[..., 3:]
    h, s, v = hsv[..., 0], hsv[..., 1], hsv[..., 2]
    i = (h * 6.0).astype('uint8')
    f = (h * 6.0) - i
    p = v * (1.0 - s)
    q = v * (1.0 - s * f)
    t = v * (1.0 - s * (1.0 - f))
    i = i % 6
    conditions = [s == 0.0, i == 1, i == 2, i == 3, i == 4, i == 5]
    rgb[..., 0] = np.select(conditions, [v, q, p, p, t, v], default=v)
    rgb[..., 1] = np.select(conditions, [v, v, v, q, p, p], default=t)
    rgb[..., 2] = np.select(conditions, [v, p, t, v, v, q], default=p)
    return rgb.astype('uint8')


# def adjust_hue_np(img, delta):
#     import numpy as np
#     img = np.array(img)
#     hsv = rgb_to_hsv(img)
#     hsv[..., 0] += delta
#     hsv[..., 0] = hsv[..., 0] % 360
#     rgb = hsv_to_rgb(hsv)
#     img = Image.fromarray(rgb.astype(np.uint8))
#     return img
# def adjust_saturation_np(img, factor):
#     import numpy as np
#     img = np.array(img)
#     hsv = rgb_to_hsv(img)
#     hsv[..., 1] *= factor
#     hsv[..., 1] = np.clip(hsv[..., 1], 0, 1)
#     rgb = hsv_to_rgb(hsv)
#     img = Image.fromarray(rgb.astype(np.uint8))
#     return img
#
# def adjust_brightness(img,factor):
#     img=ImageEnhance.Brightness(img).enhance(factor)
#     return img
# def adjust_contrast(img,factor):
#     img=ImageEnhance.Contrast(img).enhance(factor)
#     return img
# def adjust_saturation(img,factor):
#     img=ImageEnhance.Color(img).enhance(factor)
#     return img


def adjust_sharpness(img,factor):
    img=ImageEnhance.Sharpness(img).enhance(factor)
    return img
def adjust_hsv(img, h_delta=None, s_factor=None, v_factor=None):
    import numpy as np
    '''hue:0~360,s:0~1,v:0~255'''
    rgb = np.array(img)
    if h_delta is not None:
        hsv = rgb_to_hsv(rgb)
        hsv[..., 0] += h_delta
        hsv[..., 0] = hsv[..., 0] % 360
        rgb = hsv_to_rgb(hsv)
    if s_factor is not None:
        hsv = rgb_to_hsv(rgb)
        hsv[..., 1] *= s_factor
        hsv[..., 1] = np.clip(hsv[..., 1], 0, 1)
        rgb = hsv_to_rgb(hsv)
    if v_factor is not None:
        hsv = rgb_to_hsv(rgb)
        hsv[..., 2] *= v_factor
        hsv[..., 2] = np.clip(hsv[..., 2], 0, 255)
        rgb = hsv_to_rgb(hsv)
    img = Image.fromarray(rgb.astype(np.uint8))
    return img
def adjust_brightness(img, brightness_factor):
    """Adjust brightness of an Image.

    Args:
        img (PIL Image): PIL Image to be adjusted.
        brightness_factor (float):  How much to adjust the brightness. Can be
            any non negative number. 0 gives a black image, 1 gives the
            original image while 2 increases the brightness by a factor of 2.

    Returns:
        PIL Image: Brightness adjusted image.
    """
    if not _is_pil_image(img):
        raise TypeError('img should be PIL Image. Got {}'.format(type(img)))

    enhancer = ImageEnhance.Brightness(img)
    img = enhancer.enhance(brightness_factor)
    return img


def adjust_contrast(img, contrast_factor):
    """Adjust contrast of an Image.

    Args:
        img (PIL Image): PIL Image to be adjusted.
        contrast_factor (float): How much to adjust the contrast. Can be any
            non negative number. 0 gives a solid gray image, 1 gives the
            original image while 2 increases the contrast by a factor of 2.

    Returns:
        PIL Image: Contrast adjusted image.
    """
    if not _is_pil_image(img):
        raise TypeError('img should be PIL Image. Got {}'.format(type(img)))

    enhancer = ImageEnhance.Contrast(img)
    img = enhancer.enhance(contrast_factor)
    return img


def adjust_saturation(img, saturation_factor):
    """Adjust color saturation of an image.

    Args:
        img (PIL Image): PIL Image to be adjusted.
        saturation_factor (float):  How much to adjust the saturation. 0 will
            give a black and white image, 1 will give the original image while
            2 will enhance the saturation by a factor of 2.

    Returns:
        PIL Image: Saturation adjusted image.
    """
    if not _is_pil_image(img):
        raise TypeError('img should be PIL Image. Got {}'.format(type(img)))

    enhancer = ImageEnhance.Color(img)
    img = enhancer.enhance(saturation_factor)
    return img


def adjust_hue(img, hue_factor):

    """Adjust hue of an image.

    The image hue is adjusted by converting the image to HSV and
    cyclically shifting the intensities in the hue channel (H).
    The image is then converted back to original image mode.

    `hue_factor` is the amount of shift in H channel and must be in the
    interval `[-0.5, 0.5]`.

    See `Hue`_ for more details.

    .. _Hue: https://en.wikipedia.org/wiki/Hue

    Args:
        img (PIL Image): PIL Image to be adjusted.
        hue_factor (float):  How much to shift the hue channel. Should be in
            [-0.5, 0.5]. 0.5 and -0.5 give complete reversal of hue channel in
            HSV space in positive and negative direction respectively.
            0 means no shift. Therefore, both -0.5 and 0.5 will give an image
            with complementary colors while 0 gives the original image.

    Returns:
        PIL Image: Hue adjusted image.
    """
    if not(-0.5 <= hue_factor <= 0.5):
        raise ValueError('hue_factor is not in [-0.5, 0.5].'.format(hue_factor))

    if not _is_pil_image(img):
        raise TypeError('img should be PIL Image. Got {}'.format(type(img)))

    input_mode = img.mode
    if input_mode in {'L', '1', 'I', 'F'}:
        return img

    h, s, v = img.convert('HSV').split()

    np_h = np.array(h, dtype=np.uint8)
    # uint8 addition take cares of rotation across boundaries
    with np.errstate(over='ignore'):
        np_h += np.uint8(hue_factor * 255)
    h = Image.fromarray(np_h, 'L')

    img = Image.merge('HSV', (h, s, v)).convert(input_mode)
    return img


def adjust_gamma(img, gamma, gain=1):
    r"""Perform gamma correction on an image.

    Also known as Power Law Transform. Intensities in RGB mode are adjusted
    based on the following equation:

    .. math::
        I_{\text{out}} = 255 \times \text{gain} \times \left(\frac{I_{\text{in}}}{255}\right)^{\gamma}

    See `Gamma Correction`_ for more details.

    .. _Gamma Correction: https://en.wikipedia.org/wiki/Gamma_correction

    Args:
        img (PIL Image): PIL Image to be adjusted.
        gamma (float): Non negative real number, same as :math:`\gamma` in the equation.
            gamma larger than 1 make the shadows darker,
            while gamma smaller than 1 make dark regions lighter.
        gain (float): The constant multiplier.
    """
    if not _is_pil_image(img):
        raise TypeError('img should be PIL Image. Got {}'.format(type(img)))

    if gamma < 0:
        raise ValueError('Gamma should be a non-negative real number')

    input_mode = img.mode
    img = img.convert('RGB')

    gamma_map = [255 * gain * pow(ele / 255., gamma) for ele in range(256)] * 3
    img = img.point(gamma_map)  # use PIL's point-function to accelerate this part

    img = img.convert(input_mode)
    return img


def to_grayscale(img, num_output_channels=1):
    """Convert image to grayscale version of image.

    Args:
        img (PIL Image): Image to be converted to grayscale.

    Returns:
        PIL Image: Grayscale version of the image.
            if num_output_channels = 1 : returned image is single channel

            if num_output_channels = 3 : returned image is 3 channel with r = g = b
    """
    if not _is_pil_image(img):
        raise TypeError('img should be PIL Image. Got {}'.format(type(img)))

    if num_output_channels == 1:
        img = img.convert('L')
    elif num_output_channels == 3:
        img = img.convert('L')
        np_img = np.array(img, dtype=np.uint8)
        np_img = np.dstack([np_img, np_img, np_img])
        img = Image.fromarray(np_img, 'RGB')
    else:
        raise ValueError('num_output_channels should be either 1 or 3')

    return img




# Geometrical Transforms

def scale(img,scale):
    if isinstance(scale,(tuple,list)):
        scaleX,scaleY=scale
    else:
        scaleX=scaleY=scale
    w,h=img.size
    nw=int(w*scaleX)
    nh=int(h*scaleY)
    img=img.resize((nw,nh))
    return img
def resize(img,size):
    img=img.resize(size)
    return img
def shift(img,offset,fillcolor=(0,0,0)):
    fill=fillcolor
    w,h=img.size
    ofx,ofy=offset
    img = ImageChops.offset(img,ofx,ofy)
    if fill is None:
        return img
    else:
        if ofx<0:
            img.paste(fill,(ofx%w,0,w,h))
        else:
            img.paste(fill,(0,0,ofx,h))
        if ofy<0:
            img.paste(fill,(0,ofy%h,w,h))
        else:
            img.paste(fill,(0,0,w,ofy))
    return img
def translate(img,offset,fillcolor=(0,0,0)):
    return shift(img,offset,fillcolor)
def hflip(img):
    img=img.transpose(Image.FLIP_LEFT_RIGHT)
    return img
def vflip(img):
    img=img.transpose(Image.FLIP_TOP_BOTTOM)
    return img


def pad(img,dst_size,method=Image.BICUBIC, color=None):
    img=ImageOps.pad(img,dst_size,method=method,color=color)
    return img
def crop(img,box):
    img=img.crop(box)
    return img
def center_crop(img, output_size):
    if isinstance(output_size, numbers.Number):
        output_size = (int(output_size), int(output_size))
    w, h = img.size
    th, tw = output_size
    l = int(round((h - th) / 2.))
    t = int(round((w - tw) / 2.))
    r=l+int(tw)
    b=t+int(th)
    return crop(img, [l,t,r,b])
def crop_quad(img,quad,dst_size):
    '''quad:[four points clock-wise],size:target size'''
    assert isinstance(img,Image.Image)
    def invert(points):
        points=[points[0]]+reversed(points[1:])
        return points
    quad=invert(quad)
    img=img.transform(dst_size,Image.QUAD,data=quad)
    return img
def rotate(img,degree,expand=True,fill='black',translate=None):
    img=img.rotate(degree,expand=expand,fillcolor=fill,translate=translate)
    return img
def shear_x(img, degree, **kwargs):
    import math
    arc=math.radians(degree)
    factor=math.sin(arc)
    w,h=img.size
    ofx=int(h*math.sin(arc))
    dw=abs(ofx)
    img=img.transform((w+dw,h), Image.AFFINE, (1, -factor, min(0,ofx), 0, 1, 0), **kwargs)
    return img
def shear_y(img, degree, **kwargs):
    import math
    arc=math.radians(degree)
    factor=math.sin(arc)
    w,h=img.size
    ofy=int(w*math.sin(arc))
    dh=abs(ofy)
    img=img.transform((w,h+dh), Image.AFFINE, (1, 0,0, factor, 1, min(0,-ofy)), **kwargs)
    return img
def shear_xy(img,degree1,degree2):
    img=shear_x(img,degree1)
    img=shear_y(img,degree2)
    return img
def shear_yx(img,degree1,degree2):
    img=shear_x(img,degree1)
    img=shear_y(img,degree2)
    return img
def perspective_transform(img,pnts1,pnts2,dst_size,resample=Image.CUBIC,fillcolor='black'):
    import numpy
    def get_coeffs(pa, pb):
        matrix = []
        for p1, p2 in zip(pa, pb):
            matrix.append([p1[0], p1[1], 1, 0, 0, 0, -p2[0] * p1[0], -p2[0] * p1[1]])
            matrix.append([0, 0, 0, p1[0], p1[1], 1, -p2[1] * p1[0], -p2[1] * p1[1]])

        A = numpy.mat(matrix, dtype=numpy.float)
        B = numpy.array(pb).reshape(8)

        res = numpy.dot(numpy.linalg.inv(A.T * A) * A.T, B)
        return numpy.array(res).reshape(8)
    assert isinstance(img,Image.Image)
    coeffs=get_coeffs(pnts2,pnts1)
    img=img.transform(dst_size,Image.PERSPECTIVE,coeffs,resample=resample,fillcolor=fillcolor)
    return img


# Image Filtering

def gaussian_blur(img,radius=2):
    img=img.filter(ImageFilter.GaussianBlur(radius=radius))
    return img
def box_blur(img,radius=3):
    img=img.filter(ImageFilter.BoxBlur(radius=radius))
    return img
def blur(img):
    img=img.filter(ImageFilter.BLUR)
    return img
def contour(img):
    img=img.filter(ImageFilter.CONTOUR)
    return img
def edge_enhance(img):
    img=img.filter(ImageFilter.EDGE_ENHANCE)
    return img
def edge_enhance_more(img):
    img=img.filter(ImageFilter.EDGE_ENHANCE_MORE)
    return img
def emboss(img):
    img=img.filter(ImageFilter.EMBOSS)
    return img
def edge(img):
    img=img.filter(ImageFilter.FIND_EDGES)
    return img
def shapen(img):
    img=img.filter(ImageFilter.SHARPEN)
    return img
def unsharp(img,radius=5,percent=150,threshold=3):
    img=img.filter(ImageFilter.UnsharpMask(radius=radius,percent=percent,threshold=threshold))
    return img
def smooth(img):
    img=img.filter(ImageFilter.SMOOTH_MORE)
    return img
def median_filter(img,size=5):
    img = img.filter(ImageFilter.MedianFilter(size=size))
    return img
def model_filter(img,size=5):
    img=img.filter(ImageFilter.ModeFilter(size=size))
    return img
def rank_filter(img,size=5,rank=3):
    img = img.filter(ImageFilter.RankFilter(size=size,rank=rank))
    return img
def max_filter(img,size=5):
    img = img.filter(ImageFilter.MaxFilter(size=size))
    return img
def min_filter(img,size=5):
    img = img.filter(ImageFilter.MinFilter(size=size))
    return img
def equalize(img,mask=None):
    img=ImageOps.equalize(img,mask=mask)
    return img

# Image Noise
# 高斯模糊，运动模糊，高斯白噪声，椒盐噪声，poison，

def gaussian_noise(img,var=0.1):
    import numpy as np
    img=np.array(img)
    mean = 0
    sigma = var ** 0.5
    gauss = np.random.normal(mean, sigma,img.shape)
    img = img + gauss
    img = Image.fromarray(img.astype(np.uint8))
    return img
def sp_noise(img,amount=0.004):
    import numpy as np
    img = np.array(img)
    s_vs_p = 0.5
    out = np.copy(img)
    # Salt mode
    num_salt = np.ceil(amount * img.size * s_vs_p)
    coords = [np.random.randint(0, i - 1, int(num_salt))
              for i in img.shape]
    # out[coords] = 1
    out[coords] = 255

    # Pepper mode
    num_pepper = np.ceil(amount * img.size * (1. - s_vs_p))
    coords = [np.random.randint(0, i - 1, int(num_pepper))
              for i in img.shape]
    out[coords] = 0
    img=out
    img = Image.fromarray(img.astype(np.uint8))
    return img
def poisson_noise(img):
    import numpy as np
    img = np.array(img)
    vals = len(np.unique(img))
    vals = 2 ** np.ceil(np.log2(vals))
    img = np.random.poisson(img * vals) / float(vals)
    img = Image.fromarray(img.astype(np.uint8))
    return img
def speckle_noise(img):
    import numpy as np
    img = np.array(img)
    gauss = np.random.randn(*img.shape)
    img = img + img * gauss
    img=Image.fromarray(img.astype(np.uint8))
    return img


def _demo():
    from wpcv import ImageSaver
    saver=ImageSaver('/home/ars/sda5/data/tmp/0603/results',remake_dir=True)
    f='/home/ars/图片/2019011816055827.jpg'
    # f='/home/ars/图片/2020-06-03 08-44-16 的屏幕截图.png'
    img=Image.open(f).convert('RGB')
    assert isinstance(img,Image.Image)
    # x=Image.Image()
    # x.transform()
    # img=shift(img,(-50,-50),fill=None)
    # img=rotate(img,-90)
    # img=flip_horizontal(img)
    # img=flip_vertical(img)
    # img=shear_y(img,30)
    # img=shear_x(img,-30)
    # img=shear_xy(img,-30,30)
    # img=shear_x_rotate(img,-30,30)
    # img=img.filter(ImageFilter.BoxBlur(2))
    im=img.filter(ImageFilter.CONTOUR)
    saver.save(im)
    im=img.filter(ImageFilter.DETAIL)
    saver.save(im)
    im=img.filter(ImageFilter.EDGE_ENHANCE)
    saver.save(im)
    im=img.filter(ImageFilter.EDGE_ENHANCE_MORE)
    saver.save(im)
    im=img.filter(ImageFilter.EMBOSS)
    saver.save(im)
    im=img.filter(ImageFilter.FIND_EDGES)
    saver.save(im)
    im=img.filter(ImageFilter.SHARPEN)
    saver.save(im)
    im=img.filter(ImageFilter.SMOOTH_MORE)
    saver.save(im)
    im=img.filter(ImageFilter.ModeFilter(size=5))
    saver.save(im)
    im=img.filter(ImageFilter.RankFilter(size=11,rank=3))
    saver.save(im)
    im=img.filter(ImageFilter.MedianFilter(size=11))
    saver.save(im)
    im=img.filter(ImageFilter.MinFilter(size=11))
    saver.save(im)
    im=img.filter(ImageFilter.UnsharpMask(11))
    saver.save(im)
    im=equalize(img)
    saver.save(im)
    im=ImageEnhance.Color(img).enhance(0.5)
    saver.save(im)
    # im=ImageEnhance.Brightness(img).enhance(0.2)
    # saver.save(im)
    im=gaussian_noise(img)
    saver.save(im)
    im=sp_noise(img)
    saver.save(im)
    im=poisson_noise(img)
    saver.save(im)
    im=speckle_noise(img)
    saver.save(im)

    im=rotate(img,30,expand=False)
    saver.save(im)

    im.show()

if __name__ == '__main__':
    _demo()



import torch
print("PyTorch Version: ",torch.__version__)
import torch.nn as nn
import torch.optim as optim
import numpy as np
import torchvision
from torchvision import datasets, models, transforms
import matplotlib.pyplot as plt
import time
import os,glob,shutil
import copy
print("PyTorch Version: ",torch.__version__)
# print("Torchvision Version: ",torchvision.__version__)
from PIL import Image


def main():

    test(
        data_dir="/home/ars/sda5/data/chaoyuan/datasets/classify_datasets/申请表字段有无签名/val/无",
        out_dir="/home/ars/sda5/data/chaoyuan/datasets/classify_datasets/申请表字段有无签名/val_test_results",
        model_path='model.pkl',
        input_size=224,
        classes_path='trained_models/handwriting/classes.txt'
    )

def test(
    data_dir,
    out_dir=None,
    model_path = 'model.pkl',
    input_size = 224,
    classes = None,
    classes_path=None
):
    if classes_path:
        classes=open(classes_path).read().strip().split('\n')
    model, device, classes, transform=init(model_path,classes,input_size)
    test_dir(data_dir,model,out_dir,device,classes,transform)

def init(model_path,classes,input_size):


    # imgs = [data_dir + '/' + i for i in os.listdir(data_dir)]
    ###########################
    model_name = 'resnet'
    data_transforms = {
        'train': transforms.Compose([
            transforms.RandomResizedCrop(input_size),
            transforms.RandomHorizontalFlip(),
            transforms.ToTensor(),
            transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])
        ]),
        'val': transforms.Compose([
            transforms.Resize(input_size),
            transforms.CenterCrop(input_size),
            transforms.ToTensor(),
            transforms.Normalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])
        ]),
    }
    transform = data_transforms['val']
    device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")

    ###########################
    model = torch.load(model_path)
    model.to(device)
    model.eval()
    return model,device,classes,transform
###########################
os.system('killall display')
def mark_img(img,text):
    from PIL import Image,ImageDraw,ImageFont
    draw=ImageDraw.ImageDraw(img)
    font=ImageFont.truetype(font='arsocr/utils/msyh.ttf',size=16)
    draw.text((0,0),text=text,fill='red',font=font)
    return img


def test_dir(dir,model,out_dir,device,classes,transform):
    if not out_dir:
        out_dir=os.path.dirname(dir)+'/'+os.path.basename(dir)+'_test_results'
    if os.path.exists(out_dir):shutil.rmtree(out_dir)
    os.makedirs(out_dir)
    for i,f in enumerate(glob.glob(dir+'/*.jpg')):
        f2=out_dir+'/'+os.path.basename(f)
        img=load_img(f,transform)
        img=img.to(device)
        y=model(img)
        y = torch.argmax(y).cpu().int()
        y = int(y)
        cls=classes[y]
        img=Image.open(f)
        img=mark_img(img,cls)
        img.save(f2)
        print(i,f)


def load_img(f,transform):
    im = Image.open(f)
    im = transform(im).float()
    im = torch.tensor(im, requires_grad=False)
    im = im.unsqueeze(0)
    return im

def val_dir(data_dir,model,device,classes,transform):
    def iter_data():
        for cls in os.listdir(data_dir):
            cls_dir = data_dir + '/' + cls
            for f in glob.glob(cls_dir + '/*.jpg'):
                yield f, cls

    correct = 0
    total = 0
    for i, (f, cls) in enumerate(iter_data()):
        im=load_img(f,transform)
        im = im.to(device)
        y = model(im)
        y = torch.argmax(y).cpu().int()
        y = int(y)
        # print(y,len(classes))
        pred = classes[y]
        if pred == cls:
            correct += 1
        total += 1
        # print(cls==pred)
        print(i, f, y, pred, cls)

    accuracy = correct / total
    print('total: %s , accuracy : %s' % (total, accuracy))

if __name__ == '__main__':
    # test_dir(data_dir,model)

    main()


import cv2 as cv
import cv2
import numpy as np
import math
import os,shutil,glob
file_index=0
def save_to_dir(img,name=None,dir=None):
    global file_index
    name=name or '%s.jpg'%(file_index)
    dir = dir or '/home/ars/disk/work/超远/chaoyuan-part1-dev/data/tmp'
    name=dir+'/'+name
    cv2.imwrite(name,img)
    file_index+=1
# def line_detection(image):
#     gray = cv.cvtColor(image, cv.COLOR_BGR2GRAY)
#     # apertureSize做Canny时梯度窗口的大小
#     edges = cv.Canny(gray, 50, 150, apertureSize=3)
#     # 返回的是r和theta
#     lines = cv.HoughLines(edges, 1, np.pi / 180, 200)
#     for line in lines:
#         print(type(line))
#         rho, theta = line[0]
#         a = np.cos(theta)
#         b = np.sin(theta)
#         x0 = a * rho
#         y0 = b * rho
#         # 乘以1000，是根据源码乘的，通过x1、x2、y1、y2画一条直线
#         x1 = int(x0 + 1000 * (-b))
#         y1 = int(y0 + 1000 * a)
#         x2 = int(x0 - 1000 * (-b))
#         y2 = int(y0 - 1000 * a)
#         cv.line(image, (x1, y1), (x2, y2), (0, 0, 255), 2)  # 2是所画直线长度的宽
#     cv.imshow('image_lines', image)

def show(img):
    from PIL import Image
    Image.fromarray(img).show()

def detect_angle(img):
    lines=detect_lines(img)
    if not len(lines):return None
    def angle(line):
        x1,y1,x2,y2=line
        (x1,y1),(x2,y2)=sorted([[x1,y1],[x2,y2]],key=lambda p:p[0])
        ang=math.atan2(y1-y2,x2-x1)*180/math.pi
        ang=(ang+45)%90-45
        return ang
    angles=[angle(line) for line in lines]
    # print(lines)
    # print(angles)
    angle=sum(angles)/len(angles)
    return angle
def rotate(image, angle):
    # 获取图像的尺寸
    # 旋转中心
    (h, w) = image.shape[:2]
    (cx, cy) = (w / 2, h / 2)
    # 设置旋转矩阵
    M = cv2.getRotationMatrix2D((cx, cy), angle, 1.0)
    cos = np.abs(M[0, 0])
    sin = np.abs(M[0, 1])
    # 计算图像旋转后的新边界
    nW = int((h * sin) + (w * cos))
    nH = int((h * cos) + (w * sin))
    # 调整旋转矩阵的移动距离（t_{x}, t_{y}）
    M[0, 2] += (nW / 2) - cx
    M[1, 2] += (nH / 2) - cy
    return cv2.warpAffine(image, M, (nW, nH))
def rectify(img):
    # from wpkit.cv.transform.opencv import rotate
    angle=detect_angle(img)
    # print(angle)
    if abs(angle)>1:
        img=rotate(img,-angle)
    return img
def detect_lines(image,show=False):
    h,w=image.shape[:2]
    min_length=int(min(w,h)*0.2)
    gray = cv.cvtColor(image, cv.COLOR_BGR2GRAY)
    # 做Canny时梯度窗口的大小
    edges = cv.Canny(gray, 50, 150, apertureSize=3)
    # minLineLength是线段最小长度，maxLineGap是线段最大间隔，该间隔之内的认为是一条直线
    lines = cv.HoughLinesP(edges, 1, np.pi / 180, 20, minLineLength=min_length, maxLineGap=10)
    lines=[line[0] for line in lines]
    if not show:return lines
    for line in lines:
        # print(type(line))
        print(line)
        # （x1,y1）,(x2,y2)是线段的两点，只画出了一个个线段
        x1, y1, x2, y2 = line[0]
        cv.line(image, (x1, y1), (x2, y2), (0, 0, 255), 2)
    cv.imshow('image_lines', image)



def demo():
    dir='/home/ars/disk/chaoyuan/数据整理/ocr测试图片/安检报告'
    dir='/home/ars/disk/chaoyuan/testdata/0429/ocr测试图片/申请表'
    dir='/home/ars/disk/chaoyuan/数据整理/ocr测试图片/在用车检验'
    fs=glob.glob(dir+'/*.jpg')
    for i,f in enumerate(fs):
        img=cv2.imread(f)
        img=rectify(img)
        save_to_dir(img,name=os.path.basename(f))

if __name__ == '__main__':
    demo()



import os, sys, shutil, math, random, json, multiprocessing, threading
import cv2
import numpy as np
from PIL import Image, ImageEnhance, ImageFilter, ImageDraw
import abc

def pilimg(img):
    if isinstance(img,Image.Image):return img
    if isinstance(img,np.ndarray):
        if len(img.shape)==3:img=img[:,:,::-1]
    return Image.fromarray(np.array(img).astype(np.uint8))
def cv2img(img):
    if isinstance(img,Image.Image):
        img=np.array(img)
        if len(img.shape)==3:img=img[:,:,::-1]
        return img
    return img

class TransBase(object):
    def __init__(self, probability = 1.):
        super(TransBase, self).__init__()
        self.probability = probability
    @abc.abstractmethod
    def tranfun(self, inputimage):
        pass
    def process(self,inputimage):
        if np.random.random() < self.probability:
            return self.tranfun(inputimage)
        else:
            return inputimage

class RandomContrast(TransBase):
    def setparam(self, lower=0.5, upper=1.5):
        self.lower = lower
        self.upper = upper
        assert self.upper >= self.lower, "upper must be >= lower."
        assert self.lower >= 0, "lower must be non-negative."
    def tranfun(self, image):
        image = pilimg(image)
        enh_con = ImageEnhance.Brightness(image)
        return enh_con.enhance(random.uniform(self.lower, self.upper))

class RandomBrightness(TransBase):
    def setparam(self, lower=0.5, upper=1.5):
        self.lower = lower
        self.upper = upper
        assert self.upper >= self.lower, "upper must be >= lower."
        assert self.lower >= 0, "lower must be non-negative."
    def tranfun(self, image):
        image = pilimg(image)
        bri = ImageEnhance.Brightness(image)
        return bri.enhance(random.uniform(self.lower, self.upper))

class RandomColor(TransBase):
    def setparam(self, lower=0.5, upper=1.5):
        self.lower = lower
        self.upper = upper
        assert self.upper >= self.lower, "upper must be >= lower."
        assert self.lower >= 0, "lower must be non-negative."
    def tranfun(self, image):
        image = pilimg(image)
        col = ImageEnhance.Color(image)
        return col.enhance(random.uniform(self.lower, self.upper))

class RandomSharpness(TransBase):
    def setparam(self, lower=0.5, upper=1.5):
        self.lower = lower
        self.upper = upper
        assert self.upper >= self.lower, "upper must be >= lower."
        assert self.lower >= 0, "lower must be non-negative."
    def tranfun(self, image):
        image = pilimg(image)
        sha = ImageEnhance.Sharpness(image)
        return sha.enhance(random.uniform(self.lower, self.upper))

class Compress(TransBase):
    def setparam(self, lower=5, upper=85):
        self.lower = lower
        self.upper = upper
        assert self.upper >= self.lower, "upper must be >= lower."
        assert self.lower >= 0, "lower must be non-negative."
    def tranfun(self, image):
        img = cv2img(image)
        param = [int(cv2.IMWRITE_JPEG_QUALITY), random.randint(self.lower, self.upper)]
        img_encode = cv2.imencode('.jpeg', img, param)
        img_decode = cv2.imdecode(img_encode[1], cv2.IMREAD_COLOR)
        pil_img = pilimg(img_decode)
        if len(image.split())==1:
            pil_img = pil_img.convert('L')
        return pil_img

class Exposure(TransBase):
    def setparam(self, lower=5, upper=10):
        self.lower = lower
        self.upper = upper
        assert self.upper >= self.lower, "upper must be >= lower."
        assert self.lower >= 0, "lower must be non-negative."
    def tranfun(self, image):
        image = cv2img(image)
        h,w = image.shape[:2]
        x0 = random.randint(0, w)
        y0 = random.randint(0, h)
        x1 = random.randint(x0, w)
        y1 = random.randint(y0, h)
        transparent_area = (x0, y0, x1, y1)
        mask=Image.new('L', (w, h), color=255)
        draw=ImageDraw.Draw(mask)
        mask = np.array(mask)
        if len(image.shape)==3:
            mask = mask[:,:,np.newaxis]
            mask = np.concatenate([mask,mask,mask],axis=2)
        draw.rectangle(transparent_area, fill=random.randint(150,255))
        reflection_result = image + (255 - mask)
        reflection_result = np.clip(reflection_result, 0, 255)
        return pilimg(reflection_result)

class Rotate(TransBase):
    def setparam(self, lower=-5, upper=5):
        self.lower = lower
        self.upper = upper
        assert self.upper >= self.lower, "upper must be >= lower."
        # assert self.lower >= 0, "lower must be non-negative."
    def tranfun(self, image):
        image = pilimg(image)
        rot = random.uniform(self.lower, self.upper)
        trans_img = image.rotate(rot, expand=True)
        return trans_img

class Blur(TransBase):
    def setparam(self, lower=0, upper=1):
        self.lower = lower
        self.upper = upper
        assert self.upper >= self.lower, "upper must be >= lower."
        assert self.lower >= 0, "lower must be non-negative."
    def tranfun(self, image):
        image = pilimg(image)
        r=random.random()*(self.upper-self.lower)+self.lower
        image = image.filter(ImageFilter.GaussianBlur(radius=r))
        return image


class Salt(TransBase):
    def setparam(self, rate=0.02):
        self.rate = rate
    def tranfun(self, image):
        image = pilimg(image)
        num_noise = int(image.size[1] * image.size[0] * self.rate)
        # assert len(image.split()) == 1
        for k in range(num_noise):
            i = int(np.random.random() * image.size[1])
            j = int(np.random.random() * image.size[0])
            image.putpixel((j, i), int(np.random.random() * 255))
        return image


class AdjustResolution(TransBase):
    def setparam(self, max_rate=0.95,min_rate = 0.5):
        self.max_rate = max_rate
        self.min_rate = min_rate

    def tranfun(self, image):
        image = pilimg(image)
        w, h = image.size
        rate = np.random.random()*(self.max_rate-self.min_rate)+self.min_rate
        w2 = int(w*rate)
        h2 = int(h*rate)
        image = image.resize((w2, h2))
        image = image.resize((w, h))
        return image


class Crop(TransBase):
    def setparam(self, maxv=2):
        self.maxv = maxv
    def tranfun(self, image):
        img = (image)
        h,w = img.shape[:2]
        org = np.array([[0,np.random.randint(0,self.maxv)],
                        [w,np.random.randint(0,self.maxv)],
                        [0,h-np.random.randint(0,self.maxv)],
                        [w,h-np.random.randint(0,self.maxv)]],np.float32)
        dst = np.array([[0, 0], [w, 0], [0, h], [w, h]], np.float32)
        M = cv2.getPerspectiveTransform(org,dst)
        res = cv2.warpPerspective(img,M,(w,h))
        return pilimg(res)

class Crop2(TransBase):
    def setparam(self, maxv_h=4, maxv_w=4):
        self.maxv_h = maxv_h
        self.maxv_w = maxv_w
    def tranfun(self, image_and_loc):
        image, left, top, right, bottom = image_and_loc
        w, h = image.size
        left = np.clip(left,0,w-1)
        right = np.clip(right,0,w-1)
        top = np.clip(top, 0, h-1)
        bottom = np.clip(bottom, 0, h-1)
        img = cv2img(image)
        try:
            res = pilimg(img[top:bottom,left:right])
            return res
        except AttributeError as e:
            print( left, top, right, bottom)

        h = bottom - top
        w = right - left
        org = np.array([[left - np.random.randint(0, self.maxv_w), top + np.random.randint(-self.maxv_h, self.maxv_h//2)],
                        [right + np.random.randint(0, self.maxv_w), top + np.random.randint(-self.maxv_h, self.maxv_h//2)],
                        [left - np.random.randint(0, self.maxv_w), bottom - np.random.randint(-self.maxv_h, self.maxv_h//2)],
                        [right + np.random.randint(0, self.maxv_w), bottom - np.random.randint(-self.maxv_h, self.maxv_h//2)]], np.float32)
        dst = np.array([[0, 0], [w, 0], [0, h], [w, h]], np.float32)
        M = cv2.getPerspectiveTransform(org,dst)
        res = cv2.warpPerspective(img,M,(w,h))
        return pilimg(res)

class Stretch(TransBase):
    def setparam(self, max_rate = 1.2,min_rate = 0.8):
        self.max_rate = max_rate
        self.min_rate = min_rate
    def tranfun(self, image):
        image = pilimg(image)
        w, h = image.size
        rate = np.random.random()*(self.max_rate-self.min_rate)+self.min_rate
        w2 = int(w*rate)
        image = image.resize((w2, h))
        return image



import os, shutil, glob, random,time
def newdir(out_dir):
    if os.path.exists(out_dir): shutil.rmtree(out_dir)
    os.makedirs(out_dir)
def copy_files_to(files,dst,overwrite=False):
    if not os.path.exists(dst):
        os.makedirs(dst)
    for i,f in enumerate(files):
        fn=os.path.basename(f)
        f2=dst+'/'+fn
        if os.path.exists(f2):
            if os.path.samefile(f, f2):
                print("ignoring same file:", f, f2)
                continue
            if not overwrite:
                raise Exception("file %s already exists."%(f2))
            else:
                print("overwriting %s to %s ..."%(f,f2))
                os.remove(f2)
        shutil.copy(f,f2)
def split_train_val(data_dir, train_dir, val_dir, val_split=0.1, num_val=None, ext='.jpg', shuffle=True, sort=False):
    newdir(train_dir)
    newdir(val_dir)
    fs = glob.glob(data_dir + '/*' + ext)
    if sort:
        fs.sort()
    elif shuffle:
        random.shuffle(fs)
    if not num_val:
        num_val = int(len(fs) * val_split)
    val_files = fs[:num_val]
    train_files = fs[num_val:]
    copy_files_to(train_files, train_dir)
    copy_files_to(val_files, val_dir)

def split_list(lis,ratios,shuffle=True):
    if not lis:
        return None
    if len(lis)<len(ratios):
        raise Exception("List is not long enough to be split.")
    import numpy as np
    if shuffle:
        random.shuffle(lis)
    ratios=np.array(ratios)
    ratios=ratios/ratios.sum()
    nums=ratios*len(lis)
    nums=np.round(nums).astype(int)
    total=len(lis)
    splits=[]
    current_index=0
    for i,num in enumerate(nums):
        end_point=min(current_index+num,total)
        batch=lis[current_index:end_point]
        splits.append(batch)
        current_index=end_point
    return splits


def split_files(src_files,dst_dirs_and_ratios:'{"output":{"train":0.8,"val":0.2}}',shuffle=True,remake_dirs=True):
    def compress_tree(tree,root=''):
        leaves = {}
        def parse_tree(root, dic):
            for k, v in dic.items():
                p = os.path.join(root,k)
                if isinstance(v, dict):
                    parse_tree(p, v)
                else:
                    '''Now it should be a number'''
                    leaves[p] = v
        parse_tree(root=root,dic=tree)
        return leaves
    tree=compress_tree(dst_dirs_and_ratios)
    out_dirs=list(tree.keys())
    ratios=list(tree.values())
    out_file_lists=split_list(src_files,ratios,shuffle=shuffle)
    for dir,files in zip(out_dirs,out_file_lists):
        if remake_dirs and os.path.exists(dir):
            shutil.rmtree(dir)
            time.sleep(0.01)
        print('Coping %s files to %s : %s'%(len(files),dir,files))
        copy_files_to(files,dir)

def split_dir(src_dir,dst_dirs_and_ratios:'{"output":{"train":0.8,"val":0.2}}',shuffle=True,remake_dirs=True,glob_strings=['*.jpg','*.png']):
    fs=[]
    for string in glob_strings:
        fs+=glob.glob(os.path.join(src_dir,string))
    split_files(
        src_files=fs,dst_dirs_and_ratios=dst_dirs_and_ratios,shuffle=shuffle,remake_dirs=remake_dirs
    )
    print('Split dir %s finished.'%(src_dir))



def split_train_val_imagefolder(data_dir, train_dir, val_dir, val_split=0.1, num_val_cls=None, ext='.jpg', shuffle=True,
                                sort=False):
    newdir(train_dir)
    newdir(val_dir)
    for cls in os.listdir(data_dir):
        cls_dir = data_dir + '/' + cls
        train_cls_dir = train_dir + '/' + cls
        val_cls_dir = val_dir + '/' + cls
        split_train_val(cls_dir, train_dir=train_cls_dir, val_dir=val_cls_dir, val_split=val_split, num_val=num_val_cls,
                        ext=ext, shuffle=shuffle, sort=sort)
def merge_dirs(src_dirs,dst_dir):
    if  os.path.exists(dst_dir):
        shutil.rmtree(dst_dir)
        a=0
    os.makedirs(dst_dir)
    for dir in src_dirs:
        fs=glob.glob(dir+'/*')
        copy_files_to(fs,dst_dir)
if __name__ == '__main__':
    split_train_val_imagefolder(
        data_dir='/home/ars/disk/datasets/cifar-10-python/cofar10-imagenet-format',
        train_dir='/home/ars/disk/datasets/cifar-10-python/cofar10-trainval/train/raw',
        val_dir='/home/ars/disk/datasets/cifar-10-python/cofar10-trainval/val',
        # val_split=0.1
        num_val_cls=500
    )

    split_train_val_imagefolder(
        data_dir='/home/ars/disk/datasets/cifar-10-python/cofar10-trainval/train/raw',
        train_dir='/home/ars/disk/datasets/cifar-10-python/cofar10-trainval/train/unlabeled_unmerged',
        val_dir='/home/ars/disk/datasets/cifar-10-python/cofar10-trainval/train/labeled',
        # val_split=0.1
        num_val_cls=500
    )

    merge_dirs(glob.glob('/home/ars/disk/datasets/cifar-10-python/cofar10-trainval/train/unlabeled_unmerged'+'/*'),
               dst_dir='/home/ars/disk/datasets/cifar-10-python/cofar10-trainval/train/unlabeled')




from PIL import Image
import numpy as np


def rgb_to_hsv(rgb):
    # Translated from source of colorsys.rgb_to_hsv
    # r,g,b should be a numpy arrays with values between 0 and 255
    # rgb_to_hsv returns an array of floats between 0.0 and 1.0.
    rgb = rgb.astype('float')
    hsv = np.zeros_like(rgb)
    # in case an RGBA array was passed, just copy the A channel
    hsv[..., 3:] = rgb[..., 3:]
    r, g, b = rgb[..., 0], rgb[..., 1], rgb[..., 2]
    maxc = np.max(rgb[..., :3], axis=-1)
    minc = np.min(rgb[..., :3], axis=-1)
    hsv[..., 2] = maxc
    mask = maxc != minc
    hsv[mask, 1] = (maxc - minc)[mask] / maxc[mask]
    rc = np.zeros_like(r)
    gc = np.zeros_like(g)
    bc = np.zeros_like(b)
    rc[mask] = (maxc - r)[mask] / (maxc - minc)[mask]
    gc[mask] = (maxc - g)[mask] / (maxc - minc)[mask]
    bc[mask] = (maxc - b)[mask] / (maxc - minc)[mask]
    hsv[..., 0] = np.select(
        [r == maxc, g == maxc], [bc - gc, 2.0 + rc - bc], default=4.0 + gc - rc)
    hsv[..., 0] = (hsv[..., 0] / 6.0) % 1.0
    return hsv


def hsv_to_rgb(hsv):
    # Translated from source of colorsys.hsv_to_rgb
    # h,s should be a numpy arrays with values between 0.0 and 1.0
    # v should be a numpy array with values between 0.0 and 255.0
    # hsv_to_rgb returns an array of uints between 0 and 255.
    rgb = np.empty_like(hsv)
    rgb[..., 3:] = hsv[..., 3:]
    h, s, v = hsv[..., 0], hsv[..., 1], hsv[..., 2]
    i = (h * 6.0).astype('uint8')
    f = (h * 6.0) - i
    p = v * (1.0 - s)
    q = v * (1.0 - s * f)
    t = v * (1.0 - s * (1.0 - f))
    i = i % 6
    conditions = [s == 0.0, i == 1, i == 2, i == 3, i == 4, i == 5]
    rgb[..., 0] = np.select(conditions, [v, q, p, p, t, v], default=v)
    rgb[..., 1] = np.select(conditions, [v, v, v, q, p, p], default=t)
    rgb[..., 2] = np.select(conditions, [v, p, t, v, v, q], default=p)
    return rgb.astype('uint8')


def adjust_hue(img, delta):
    img = np.array(img)
    hsv = rgb_to_hsv(img)
    hsv[..., 0] += delta
    hsv[..., 0] = hsv[..., 0] % 360
    rgb = hsv_to_rgb(hsv)
    img = Image.fromarray(rgb.astype(np.uint8))
    return img


def adjust_saturation(img, factor):
    img = np.array(img)
    hsv = rgb_to_hsv(img)
    hsv[..., 1] *= factor
    hsv[..., 1] = np.clip(hsv[..., 1], 0, 1)
    rgb = hsv_to_rgb(hsv)
    img = Image.fromarray(rgb.astype(np.uint8))
    return img


def adjust_hsv(img, h_delta=None, s_factor=None, v_factor=None):
    '''hue:0~360,s:0~1,v:0~255'''
    rgb = np.array(img)
    if h_delta is not None:
        hsv = rgb_to_hsv(rgb)
        hsv[..., 0] += h_delta
        hsv[..., 0] = hsv[..., 0] % 360
        rgb = hsv_to_rgb(hsv)
    if s_factor is not None:
        hsv = rgb_to_hsv(rgb)
        hsv[..., 1] *= s_factor
        hsv[..., 1] = np.clip(hsv[..., 1], 0, 1)
        rgb = hsv_to_rgb(hsv)
    if v_factor is not None:
        hsv = rgb_to_hsv(rgb)
        hsv[..., 2] *= v_factor
        hsv[..., 2] = np.clip(hsv[..., 2], 0, 255)
        rgb = hsv_to_rgb(hsv)
    img = Image.fromarray(rgb.astype(np.uint8))
    return img


if __name__ == '__main__':
    green_hue = (180 - 78) / 360.0
    red_hue = (180 - 180) / 360.0

    img = Image.open('/home/ars/图片/0.jpg')
    # img=adjust_hue(img,100)
    # img=adjust_saturation(img,0.02)
    img = adjust_hsv(img, 100, 0.98, 0.7)
    img.show()



import setuptools
import os,glob
with open("README.md", "r") as fh:
    long_description = fh.read()

def get_version():
    import os
    vf='version.txt'
    maxversions=[9,9,9,9,9,9,9]
    def add_version(version,index=-1):
        if index==-1:index=len(version)-1
        version[index]+=1
        if version[index]>maxversions[len(version)-index-1]:
            version[index]=0
            if index==0:
                raise Exception('Max version number reached. %s' %(version))
            return add_version(version,index-1)
        else:
            return version
    if not os.path.exists(vf):
        raise Exception('version.txt not found or  is invalid')
        # f=open(vf,'w')
        # f.write(str(249))
        # f.close()
    with open(vf,'r') as f:
        numbers=f.read().strip()
        if numbers=='':
            raise Exception('version.txt not found or  is invalid')
        version=numbers.split('.')
        version=[int(x) for x in version]
        version=add_version(version)
        version='.'.join([str(x) for x in version])

    with open(vf,'w') as f:
        f.write(version)
    return version
version=get_version()
print("version:",version)
setuptools.setup(
    executable=True,
    name="wpcv", # Replace with your own username
    version=version,
    author="Wang Pei",
    author_email="1535376447@qq.com",
    description="CV Devkit of Wang Pei",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Peiiii/wpcv",
    packages=setuptools.find_packages(),
    package_dir={'wpcv':'wpcv'},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.0',
)



'''Parameters
----------
image : ndarray
    Input image data. Will be converted to float.
mode : str
    One of the following strings, selecting the type of noise to add:

    'gauss'     Gaussian-distributed additive noise.
    'poisson'   Poisson-distributed noise generated from the data.
    's&p'       Replaces random pixels with 0 or 1.
    'speckle'   Multiplicative noise using out = image + n*image,where
                n is uniform noise with specified mean & variance.

'''

import numpy as np
import os
import cv2


def noisy(noise_typ, image):
    if noise_typ == "gauss":
        row, col, ch = image.shape
        mean = 0
        var = 0.1
        sigma = var ** 0.5
        gauss = np.random.normal(mean, sigma, (row, col, ch))
        # gauss = gauss.reshape(row, col, ch)
        noisy = image + gauss
        return noisy
    elif noise_typ == "s&p":
        row, col, ch = image.shape
        s_vs_p = 0.5
        amount = 0.004
        out = np.copy(image)
        # Salt mode
        num_salt = np.ceil(amount * image.size * s_vs_p)
        coords = [np.random.randint(0, i - 1, int(num_salt))
                  for i in image.shape]
        out[coords] = 1

        # Pepper mode
        num_pepper = np.ceil(amount * image.size * (1. - s_vs_p))
        coords = [np.random.randint(0, i - 1, int(num_pepper))
                  for i in image.shape]
        out[coords] = 0
        return out
    elif noise_typ == "poisson":
        vals = len(np.unique(image))
        vals = 2 ** np.ceil(np.log2(vals))
        noisy = np.random.poisson(image * vals) / float(vals)
        return noisy
    elif noise_typ == "speckle":
        row, col, ch = image.shape
        gauss = np.random.randn(row, col, ch)
        gauss = gauss.reshape(row, col, ch)
        noisy = image + image * gauss
        return noisy



import numpy as np
from PIL import Image

def quad_as_rect(quad):
    if quad[0] != quad[2]: return False
    if quad[1] != quad[7]: return False
    if quad[4] != quad[6]: return False
    if quad[3] != quad[5]: return False
    return True

def quad_to_rect(quad):
    assert(len(quad) == 8)
    assert(quad_as_rect(quad))
    return (quad[0], quad[1], quad[4], quad[3])

def rect_to_quad(rect):
    assert(len(rect) == 4)
    return (rect[0], rect[1], rect[0], rect[3], rect[2], rect[3], rect[2], rect[1])

def shape_to_rect(shape):
    assert(len(shape) == 2)
    return (0, 0, shape[0], shape[1])

def griddify(rect, w_div, h_div):
    w = rect[2] - rect[0]
    h = rect[3] - rect[1]
    x_step = w / float(w_div)
    y_step = h / float(h_div)
    y = rect[1]
    grid_vertex_matrix = []
    for _ in range(h_div + 1):
        grid_vertex_matrix.append([])
        x = rect[0]
        for _ in range(w_div + 1):
            grid_vertex_matrix[-1].append([int(x), int(y)])
            x += x_step
        y += y_step
    grid = np.array(grid_vertex_matrix)
    return grid

def distort_grid(org_grid, max_shift):
    new_grid = np.copy(org_grid)
    x_min = np.min(new_grid[:, :, 0])
    y_min = np.min(new_grid[:, :, 1])
    x_max = np.max(new_grid[:, :, 0])
    y_max = np.max(new_grid[:, :, 1])
    new_grid += np.random.randint(- max_shift, max_shift + 1, new_grid.shape)
    new_grid[:, :, 0] = np.maximum(x_min, new_grid[:, :, 0])
    new_grid[:, :, 1] = np.maximum(y_min, new_grid[:, :, 1])
    new_grid[:, :, 0] = np.minimum(x_max, new_grid[:, :, 0])
    new_grid[:, :, 1] = np.minimum(y_max, new_grid[:, :, 1])
    return new_grid

def grid_to_mesh(src_grid, dst_grid):
    assert(src_grid.shape == dst_grid.shape)
    mesh = []
    for i in range(src_grid.shape[0] - 1):
        for j in range(src_grid.shape[1] - 1):
            src_quad = [src_grid[i    , j    , 0], src_grid[i    , j    , 1],
                        src_grid[i + 1, j    , 0], src_grid[i + 1, j    , 1],
                        src_grid[i + 1, j + 1, 0], src_grid[i + 1, j + 1, 1],
                        src_grid[i    , j + 1, 0], src_grid[i    , j + 1, 1]]
            dst_quad = [dst_grid[i    , j    , 0], dst_grid[i    , j    , 1],
                        dst_grid[i + 1, j    , 0], dst_grid[i + 1, j    , 1],
                        dst_grid[i + 1, j + 1, 0], dst_grid[i + 1, j + 1, 1],
                        dst_grid[i    , j + 1, 0], dst_grid[i    , j + 1, 1]]
            dst_rect = quad_to_rect(dst_quad)
            mesh.append([dst_rect, src_quad])
    return mesh

im = Image.open('./old_driver/data/train/c0/img_292.jpg')
dst_grid = griddify(shape_to_rect(im.size), 4, 4)
src_grid = distort_grid(dst_grid, 50)
mesh = grid_to_mesh(src_grid, dst_grid)
im = im.transform(im.size, Image.MESH, mesh)
im.show()